set saveimagedir ""
set saveimagelist ""
set oldsaveformat TIFF
set oldsavegformat AVW

set SaveAsImageList {TIFF Raw AVW AVW_SPM_TAL VEST EPS JPEG}
set SaveAsGroupList {AVW}
set TransformList {IntoTalairachSpace GenericReslice GenericResample MotionCorrectionReslice AlignWarpReslice CopyVOIReslice UserTransformation} 

set PLATFORM [exec $PXHOME/bin/platform]

proc OpenFolder { w } {
    SelectFile $w "Open Folder" "Open" OpenFolder FolderDirHistory
}

proc SaveImageAs { w } {
    SelectFile $w "Save Image As" "Save" SaveImageAs DirectoryHistory
}

proc SaveGroupAs { w } {
    SelectFile $w "Save Group As" "Save" SaveGroupAs DirectoryHistory
}

proc SaveFolderAs { w } { 
    SelectFile $w "Save Folder As" "Save" SaveFolderAs FolderDirHistory
}

proc ApplyParadigm {w} {
    SelectFile $w "Apply Paradigm" "Ok" ApplyParadigm DirectoryHistory
}

proc ApplyShadowTransform {w} {
    SelectFile $w "Apply Shadow Transform" "Ok" ApplyShadowTransform DirectoryHistory
}

proc SaveShadowTransform {w} {
    SelectFile $w "Save Shadow Transform" "Save" SaveShadowTransform DirectoryHistory
}

proc LoadROITemplate {w} {
    SelectFile $w "Load ROI Template" "Load" LoadROITemplate DirectoryHistory
}

proc SaveROITemplate {w} {
    SelectFile $w "Save ROI Template" "Save" SaveROITemplate DirectoryHistory
}

proc ReadEPSF {w} {
    SelectFile $w "Read EPSF" "Read" ReadEPSF DirectoryHistory
}

proc SelectHeadHat {w} {
    SelectFile $w "Select Head Hat File" "Select" SelectHeadHat DirectoryHistory
}

proc SaveGroupAsMenu {w} {
    $w add command AVW -label "AVW" 
    $w add command AVW_SPM_TAL -label "AVW (MEDx Tal To SPM)"
    $w add command Dicom -label "Dicom"
}

# Procedure:	GetHistoryProperties
#
# Usage:	GetHistoryProperties widget target property
#
# Summary:	Retrieve property from target
#
# ie GetHistoryProperties .foo Session ImportImageHistory
#
# The widget here is the file box itself!
#
proc GetHistoryProperties { w target property size } {
    global $property

    set histlist [$w subwidget filter ] 

    Message:GetProperties DEFAULTS.$target $property prop
    set list [ PxGet $prop $property "./*" ]

    if { [llength $list ] == 0 } {
	set dir "~"
	set pattern "*"
    } else {
	# Use directory and pattern from the string, otherwise
	# the box always puts /* at the end of the string.
	regexp "(.*)/(.*)" [lindex $list 0 ] m dir pattern
	foreach i $list {
	    $histlist appendhistory $i
	}
    }

    $w config -directory $dir -pattern $pattern
}


# Procedure:	SetHistoryProperties
#
# Usage:	SetHistoryProperties widget target property size
#
# Summary:	Set property in target
# ie SetHistoryProperties .foo Session ImportImageHistory
proc SetHistoryProperties { w target property } { 
    global $property
    # convert a list to an array of strings
    set pxlist ""

    set plist [GetListItems $w]

    # cut list down to requested size
    set len [ min 10 [ llength $plist ] ]
    set plist [ lrange $plist 0 $len ]
    
    foreach dir "$plist" {
#	set dir [string trimright $dir "/*"]
	set pxlist "$pxlist \($dir\)"
    }

    Message:SetProperties DEFAULTS.$target "
        $property \[ $pxlist \]"
}

# luniq - remove duplicates from the list
proc luniq { list } {
    set newlist { }

    # search for every entry in the list, extract it, and put it
    # in the new list.
    foreach i $list {
	lappend $newlist [lindex $list [ lsearch -exact $list $i ]]
    }
    return $newlist
}

# Procedure:	SelectFile
#
# Usage:	SelectFile widget remotefileid filedialogtype
#
# Summary:	
#
proc SelectFile {w type okbuttontext help history } {
    global selectfile 
    global saveimagedir
    global saveimagelist
    global oldsaveformat oldsavegformat currenttype
    global SaveAsImageList SaveAsGroupList TransformList
    global PLATFORM

    if [winfo exists $w] {
	wm deiconify $w
	raise $w
	Message:DialogPosted $w
	return
    }

    set currenttype $type
    set selectfile(history) $history

    # Remember this funcname comes from remove blanks from the type
    # and the apply function is based on that.
    regsub -all " " $type "" selectfile($w,funcname)

    toplevel $w 
    wm title $w $type
    wm iconname $w $type

    # ---- Box ----

if { 0 } {
    if { $saveimagedir == "" } {
	set dir ""
	set pattern "*"
    } else {
	regexp "(.*)/(.*)" $saveimagedir m dir pattern
    }
}
    if { $type == "Open Folder" || $type == "Save Folder As"} {
        tixFolderSelectBox $w.box \
	    -command "SelectFile:doit $w"
#	    -directory $dir
    } else {
        tixFileSelectBox $w.box \
	    -command "SelectFile:doit $w"
#	    -directory $dir
    }

    #------------- Button Frame --------------

    frame $w.btns
    frame $w.btns.b -relief raised -borderwidth 1
    button $w.ok -command "SelectFile:ok $w" \
	-text $okbuttontext -width 5
    bind $w.ok <Return> "$w.box invoke"
    button $w.apply -text "Apply" -width 5 \
	-command "SelectFile:ap $w" 
    bind $w.apply <Return> "$w.apply invoke"
    if {$type == "Apply Shadow Transform"} {
	button $w.reset -text "Reset Pixel Report" -width 15 \
	    -command "SelectFile:reset $w" 
	bind $w.reset <Return> "$w.reset invoke"
    }
    button $w.filter -command "$w.box filter" \
	-text "Filter" -width 5
    bind $w.filter <Return> {
	[winfo toplevel %W].box filter}
    button $w.cancel -command "DestroyDialog SelectFile $w" \
	-text "Cancel" -width 5
    bind $w.cancel <Return> {
	DestroyDialog SelectFile [winfo toplevel %W]}
    button $w.help   -command "WebHelp dialogs/$help" \
	-text "Help" -width 5

    pack $w.btns.b -fill x -padx 5 -pady 10
    if {$type == "Apply Shadow Transform"} {
	pack $w.ok $w.filter $w.apply $w.reset $w.cancel $w.help -in $w.btns.b \
	    -side left -expand yes -padx 5 -pady 10 -fill y
    } else {
	pack $w.ok $w.filter $w.apply $w.cancel $w.help -in $w.btns.b \
	    -side left -expand yes -padx 5 -pady 10 -fill y
    }

    GetHistoryProperties $w.box Session $history 10

    #------------- Upper Frame --------------
    if {$type == "Save Image As"} {
	wm geometry $w 500x510
	frame $w.uf
	tixOptionMenu $w.uf.o -label "File Format" \
	    -variable selectfile($w,format) \
	    -options {
		label.width 11
		label.anchor w
		menubutton.width 15
	}
	tixOptionMenu $w.byteswap -label "File Byte Order" \
	    -variable selectfile($w,byteswap) \
	    -options {
		label.width 14
		label.anchor w
		menubutton.width 14
	}
        OpMenuList $w.byteswap {
		{ NoSwap	"No Swap" }
		{ Swap		"Swap" }
		{ MSB		"MSB First" }
		{ LSB		"LSB First" }	
        }
	SaveImageAsMenu $w.uf.o 
	
	if { $oldsaveformat == "" } {
	    set oldsaveformat TIFF
	}

	set validformat [lsearch -exact $SaveAsImageList $oldsaveformat]

	if { $validformat == -1 } {
	    set oldsaveformat TIFF
	}

	set selectfile($w,format) $oldsaveformat

	$w.uf.o configure -value $selectfile($w,format)
        if { [string compare $selectfile($w,format) "AVW"] == 0 } {
    	   if {$PLATFORM != "AXP_3" } {
	       pack $w.uf.o $w.byteswap -in $w.uf -side right -padx 8 -pady 3
	   } else {
	       pack forget $w.byteswap
	       pack $w.uf.o -in $w.uf -padx 8 -pady 3
	   }
        } 


	$w.box.f2 paneconfigure directory -size 250

	$w.uf.o configure -command "SaveAsImageCommand $w" 



	set histlist [$w.box subwidget filter] 
	foreach i $saveimagelist {
	    set i [string trimright $i "*"]
	    set i [string trimright $i "/"]
	    $histlist appendhistory $i
	}

	pack $w.uf -pady 3 -fill x
	pack $w.uf.o -in $w.uf -side top -padx 3 -pady 3
	pack $w.box -side top -expand yes -fill both -padx 3
	pack $w.box $w.btns -fill both

	set selectfile($w,disposition) ""

    } elseif {$type == "Save Group As"} {
	wm geometry $w 500x510
	frame $w.uf
	tixOptionMenu $w.uf.o -label "File Format" \
	    -variable selectfile($w,format) \
	    -options {
		label.width 11
		label.anchor w
		menubutton.width 15
	}
	tixOptionMenu $w.byteswapg -label "File Byte Order" \
	    -variable selectfile($w,byteswapg) \
	    -options {
		label.width 14
		label.anchor w
		menubutton.width 14
	}
        OpMenuList $w.byteswapg {
		{ NoSwap	"No Swap" }
		{ Swap		"Swap" }
		{ MSB		"MSB First" }
		{ LSB		"LSB First" }	
        }
	SaveGroupAsMenu $w.uf.o 
	
	if { $oldsavegformat == "" } {
	    set oldsavegformat AVW
	}

	set validgformat [lsearch -exact $SaveAsGroupList $oldsavegformat]
	if { $validgformat == -1 } {
	    set oldsavegformat AVW
	}

	set selectfile($w,format) $oldsavegformat
	if { $selectfile($w,format) == "AVW" } {
    	   if {$PLATFORM != "AXP_3" } {
	       pack $w.byteswapg $w.uf.o -in $w.uf -side right -padx 6 -pady 3
	   } else {
	       pack forget $w.byteswapg
	       pack $w.uf.o -in $w.uf -padx 6 -pady 3
	   }
	}

	$w.uf.o configure -value $selectfile($w,format)

	$w.box.f2 paneconfigure directory -size 250
	
	$w.uf.o configure -command "SaveAsGroupCommand $w"

	set histlist [$w.box subwidget filter] 
	foreach i $saveimagelist {
	    set i [string trimright $i "*"]
	    set i [string trimright $i "/"]
	    $histlist appendhistory $i
	}

	pack $w.uf -pady 3 -fill x
	pack $w.uf.o -side top -padx 3 -pady 3
	pack $w.box -side top -expand yes -fill both -padx 3
	pack $w.box $w.btns -fill both

	set selectfile($w,disposition) ""

    } elseif {$type == "Open As Group"} {
	wm geometry $w 500x500

	$w.box.f2 paneconfigure directory -size 250

	set filewidget [ $w.box subwidget filelist subwidget listbox ]
	    $filewidget configure -selectmode extended

	pack $w.box -side top -expand yes -fill both -padx 3
	pack $w.box $w.btns -fill both

	set selectfile($w,format) ""
	set selectfile($w,disposition) ""

    } elseif {$type == "Apply Shadow Transform"} {
	wm geometry $w 580x450

	frame $w.uf

	tixOptionMenu $w.uf.dir -label "Direction" \
	    -variable selectfile($w,direction) \
	    -options {
		label.width 12
		label.anchor w
		menubutton.width 20
	}
        $w.uf.dir add command Forward -label "Forward"
        $w.uf.dir add command Inverse -label "Inverse"

	tixOptionMenu $w.uf.o -label "Transform" \
	    -variable selectfile($w,format) \
	    -command "SelectFile:TransformUpdate $w" \
	    -options {
		label.width 12
		label.anchor w
		menubutton.width 20
	}

        $w.uf.o add command MotionCorrectionReslice -label "AlignLinearReslice"
        $w.uf.o add command AlignWarpReslice -label "AlignWarpReslice"
        $w.uf.o add command CopyVOIReslice -label "CopyVOIReslice"
        $w.uf.o add command GenericReslice -label "GenericReslice"
        $w.uf.o add command GenericResample -label "GenericResample"
        $w.uf.o add command IntoTalairachSpace -label "IntoTalairachSpace"
        $w.uf.o add command UserTransformation -label "UserTransformation"
	$w.uf.o configure -value MotionCorrectionReslice

	set selectfile($w,pixelreport) 0
	checkbutton $w.pixel -text "Use For Pixel Reporting" \
		-variable selectfile($w,pixelreport)

	$w.box.f2 paneconfigure directory -size 250

	pack $w.uf -pady 3 -fill x
	pack $w.uf.o -side left -padx 3 -pady 3
	pack $w.uf.dir -side left -padx 3 -pady 3
	pack $w.pixel -side top -padx 3 -pady 3
	pack $w.box -side top -expand yes -fill both -padx 3
	pack $w.box $w.btns -fill both

	set selectfile($w,disposition) ""

    } elseif {$type == "Save Folder As"} {
	wm geometry $w 500x450

	$w.box.f2 paneconfigure directory -size 250

	set selectfile($w,format) ""
	set selectfile($w,disposition) ""

	pack $w.box -side top -expand yes -fill both -padx 3
	pack $w.btns -fill both -pady 3
	tixBalloon $w.bhelp 

    } else {
	wm geometry $w 500x450
	$w.box.f2 paneconfigure directory -size 250

	set selectfile($w,format) ""
	set selectfile($w,disposition) ""

	pack $w.box -side top -expand yes -fill both -padx 3
	pack $w.btns -fill both
    }



    bind $w <Control-c> {
	DestroyDialog SelectFile [winfo toplevel %W]
    }

    Message:DialogPosted $w
}

proc SaveAsImageCommand { win { format } } {
    global saveimage
    global selectfile
    global saveasdicom
    global dicom
    global dcm
    global PLATFORM

    set saveasdicom(savemode) "image"

    if { [string compare $format "Dicom"] == 0 } {

	#---------- DICOM Dictionary ------------------------------
	DICOM:dict

	DestroyDialog SelectFile .saveimageas

	set newwin .saveasdicom
	if [winfo exists $newwin] {
	    wm deiconify $newwin
	    raise $newwin
	    return
	}
	
	set w [toplevel $newwin]
	wm title $w "Save As DICOM"
	wm iconname $w "Save As DICOM"
#	wm minsize $w 500 200
	
	SaveAsDICOM:setdefaults $w

	Message:ExpressPropertyInterest SaveAsDICOM:update $w \
	CurrentImage {
	    Name FileName Width Height Slices Depth ImageFileFormat 
	    PixelEncoding ComplexMode Temporary?
	    UserMatrix DistanceUnits
	    IntensityGain IntensityOffset PixelIntensityUnits
	    ShowCaptions? CaptionColor SliceHighlight?
	    AnatomicAxes
	}


	frame $w.f
	tixFileEntry $w.f.dicomdir \
	    -label "DICOM Directory" \
	    -variable saveasdicom(dir) \
	    -dialogtype tixDirSelectDialog \
	    -options {
		label.width 22
	    }

	set saveasdicom(iodlist) [list MR CT CR]
	set iodlength [llength $saveasdicom(iodlist)]

	tixOptionMenu $w.f.iod -label "DICOM Object"  \
	    -dynamicgeometry true \
	    -variable saveasdicom(iod) \
	    -options {
		label.width 22
		menubutton.width 4
	    }

	for { set i 0 } { $i < $iodlength } {incr i} {
	    set iod [lindex $saveasdicom(iodlist) $i]
	    $w.f.iod add command $iod -label $iod 
	}

	tixOptionMenu $w.f.filelength -label "DICOM Filename Length" \
	    -dynamicgeometry true \
	    -variable saveasdicom(filelength) \
	    -options {
		label.width 22
		menubutton.width 6
	    }

	$w.f.filelength add command Short -label "Short" 
	$w.f.filelength add command Long -label "Long" 

	frame $w.btns
	frame $w.btns.b -relief raised -borderwidth 1
	button $w.ok -command "SaveAsDICOM:ok $w" \
	    -text "Save" -width 5
	bind $w.ok <Return> "$w.box invoke"
	button $w.apply -text "Apply" -width 5 \
	    -command "SaveAsDICOM:apply $w" 
	bind $w.apply <Return> "$w.apply invoke"
	button $w.cancel -command "SaveAsDICOM:destroy $w" \
	    -text "Cancel" -width 5
	bind $w.cancel <Return> {
	    SaveAsDICOM:destroy [winfo toplevel %W]}
	set help "SaveImageAs"
	button $w.help   -command "WebHelp dialogs/$help" \
	    -text "Help" -width 5

	pack $w.btns.b -fill x -padx 5 -pady 10
	pack $w.ok $w.apply $w.cancel $w.help -in $w.btns.b \
	    -side left -expand yes -padx 5 -pady 10 -fill y
	pack $w.f.dicomdir $w.f.iod $w.f.filelength -side top -fill both
	pack $w.f $w.btns -side top -fill both -pady 5

    } elseif { [string compare $format "AVW"] == 0 } {
	pack forget $win.uf.o
    	if {$PLATFORM != "AXP_3" } {
	    pack $win.uf.o $win.byteswap -in $win.uf -side left -padx 8 -pady 3
	} else {
	    pack forget $win.byteswap
	    pack $win.uf.o -in $win.uf -padx 8 -pady 3
	}
    } else {
	pack forget $win.byteswap
	pack forget $win.uf.o
	pack $win.uf.o -in $win.uf -side top -padx 3 -pady 3
    }
}


proc SaveAsGroupCommand { win { format } } {
    global saveimage
    global selectfile
    global saveasdicom
    global dicom
    global dcm
    global PLATFORM


    set saveasdicom(savemode) "group"

    if { [string compare $format "Dicom"] == 0 } {

	#---------- DICOM Dictionary ------------------------------
	DICOM:dict

	DestroyDialog SelectFile .savegroupas

	set newwin .saveasdicom
	if [winfo exists $newwin] {
	    wm deiconify $newwin
	    raise $newwin
	    return
	}
	
	set w [toplevel $newwin]
	wm title $w "Save As DICOM"
	wm iconname $w "Save As DICOM"
	
	SaveAsDICOM:setdefaults $w

	frame $w.f
	tixFileEntry $w.f.dicomdir \
	    -label "DICOM Directory" \
	    -variable saveasdicom(dir) \
	    -dialogtype tixDirSelectDialog \
	    -options {
		label.width 22
	    }


	set saveasdicom(iodlist) [list MR CT CR]
	set iodlength [llength $saveasdicom(iodlist)]

	tixOptionMenu $w.f.iod -label "DICOM Object"  \
	    -dynamicgeometry true \
	    -variable saveasdicom(iod) \
	    -options {
		label.width 22
		menubutton.width 4
	    }

	for { set i 0 } { $i < $iodlength } {incr i} {
	    set iod [lindex $saveasdicom(iodlist) $i]
	    $w.f.iod add command $iod -label $iod 
	}

	tixOptionMenu $w.f.filelength -label "DICOM Filename Length" \
	    -dynamicgeometry true \
	    -variable saveasdicom(filelength) \
	    -options {
		label.width 22
		menubutton.width 6
	    }

	$w.f.filelength add command Short -label "Short" 
	$w.f.filelength add command Long -label "Long" 

	frame $w.btns
	frame $w.btns.b -relief raised -borderwidth 1
	button $w.ok -command "SaveAsDICOM:ok $w" \
	    -text "Save" -width 5
	bind $w.ok <Return> "$w.box invoke"
	button $w.apply -text "Apply" -width 5 \
	    -command "SaveAsDICOM:apply $w" 
	bind $w.apply <Return> "$w.apply invoke"
	button $w.cancel -command "SaveAsDICOM:destroy $w" \
	    -text "Cancel" -width 5
	bind $w.cancel <Return> {
	    SaveAsDICOM:destroy [winfo toplevel %W]}
	set help "SaveImageAs"
	button $w.help   -command "WebHelp dialogs/$help" \
	    -text "Help" -width 5

	pack $w.btns.b -fill x -padx 5 -pady 10
	pack $w.ok $w.apply $w.cancel $w.help -in $w.btns.b \
	    -side left -expand yes -padx 5 -pady 10 -fill y
	pack $w.f.dicomdir $w.f.iod $w.f.filelength -side top -fill both
	pack $w.f $w.btns -side top -fill both -pady 5

    } elseif { [string compare $format "AVW"] == 0 } {
	pack forget $win.uf.o
    	if {$PLATFORM != "AXP_3" } {
	    pack $win.uf.o $win.byteswapg -in $win.uf -side left -padx 8 -pady 3
	} else {
	    pack forget $win.byteswapg
	    pack $win.uf.o -in $win.uf -padx 6 -pady 3
	}
    } else {
	pack forget $win.byteswapg
	pack forget $win.uf.o
	pack $win.uf.o -in $win.uf -side top -padx 3 -pady 3
    }

}

# Procedure:	SelectFile:doit
#
# Usage:	SelectFile:doit widget filename
#
# Summary:	Invoke callback for fileSelectBox
#
proc SelectFile:doit {w filename} {
    global saveimage

    SelectFile:apply $w $filename
}

# Procedure:	SelectFile:TransformUpdate
#
# Usage:	SelectFile:TransformUpdate widget type
#
# Summary:	updates direction widget based on type
#
proc SelectFile:TransformUpdate { w type } {
    global saveimage

    if {$type == "AlignWarpReslice" } {
	$w.uf.dir config -state disabled
    } else {
	$w.uf.dir config -state normal
    }
}

# Procedure:	SelectFile:ok
#
# Usage:	SelectFile:ok widget
#
# Summary:	Apply callback: sends the contents of the dialog box to the
#		remote file id.
#
proc SelectFile:ok {w} {
    global saveimage

    Message:OnNextCompletion "DestroyDialog SelectFile $w"
    $w.box invoke
}


# Procedure:	SelectFile:ap
#
# Usage:	SelectFile:ap widget filename
#
# Summary:	Apply callback: sends the contents of the dialog box to the
#		remote file id.
#
proc SelectFile:ap {w} {
    global saveimage

    $w.box invoke
}

# Procedure:	SelectFile:reset
#
# Usage:	SelectFile:reset widget filename
#
# Summary:	Reset Pixel Reporting callback: sends message to rest pixel
#		reporting.
#
proc SelectFile:reset {w} {
    global saveimage

    Message:InvokeFunction ResetPixelShadowTransform ""
}


# Procedure:	SelectFile:apply
#
# Usage:	SelectFile:apply widget filename
#
# Summary:	Apply callback: sends the contents of the dialog box to the
#		remote file id.
#
proc SelectFile:apply {w filename} {
    global selectfile
    global rawimage

    switch $selectfile($w,funcname) {

	"OpenFolder" {
	    Message:InvokeFunction OpenFolder "FileName ($filename)"
	}

	"SaveFolderAs" {
	    if { ! [ catch { set cookie $selectfile($w,cookie) } ] } {
		Message:InvokeFunction SaveFolder \
		    "FileName ($filename) Folder $cookie"
	    } else {
		Message:InvokeFunction SaveFolder \
		    "FileName ($filename)"
	    }
	}

	"SaveImageAs"  {
	    set selectfile($w,byteordertype) [$w.byteswap.frame.menubutton cget -text]
	    if { $selectfile($w,format) == "AutoDetectFormat" } {
		set format ""
	    } else {
		set format "FileFormat $selectfile($w,format)"
	    }
	    set bytetype "N"

#	The MessageHandler is not accepting more than two arguments Yet I need
#	to pass the specified ByteOrder in the message somehow. So, for AVW,
#	the filename is cpended with the following code letters - TALIN
#		NoSwap - N;  Swap - S;  MSB - M;  LSB - L

	    if { $selectfile($w,format) == "AVW" } {
		set bytetype "L"
       		if { $selectfile($w,byteordertype) == "No Swap" } { 
			set bytetype "N"	
    		} elseif {$selectfile($w,byteordertype)  == "Swap"} {
			set bytetype "S"	
    		} elseif {$selectfile($w,byteordertype) == "MSB First"} {
			set bytetype "M"	
		}  	
	    }

	    Message:InvokeFunction SaveImage "
                FileName ($filename)
		ByteFlag ($bytetype)
		$format
	    "
	}

	"SaveGroupAs"  {
	    set selectfile($w,byteordertype) [$w.byteswapg.frame.menubutton cget -text]
	    set format "FileFormat $selectfile($w,format)"
	    set bytetype "N"


#	The MessageHandler is not accepting more than two arguments Yet I need
#	to pass the specified ByteOrder in the message somehow. So, for AVW,
#	the filename is cpended with the following code letters - TALIN
#		NoSwap - N;  Swap - S;  MSB - M;  LSB - L

	    if { $selectfile($w,format) == "AVW" } {
		set bytetype "L"
       		if { $selectfile($w,byteordertype) == "No Swap" } { 
			set bytetype "N"	
    		} elseif {$selectfile($w,byteordertype)  == "Swap"} {
			set bytetype "S"	
    		} elseif {$selectfile($w,byteordertype) == "MSB First"} {
			set bytetype "M"	
		}  	
	    }

	    Message:InvokeFunction Save3DGroup "
                FileName ($filename)
		ByteFlag ($bytetype)
		$format
	    "
	}

	"ReadEPSF" {
	    Message:InvokeFunction ImportEPSGraphic "
		FileName ($filename) "
	}

	"ApplyParadigm" {
	    Message:InvokeFunction ApplyParadigmFile "
		/FileName	($filename)"
	}

	"ApplyShadowTransform" {
	    if $selectfile($w,pixelreport) {
		Message:InvokeFunction ApplyPixelShadowTransform "
		    /transformsource	($filename)
		    /Direction		$selectfile($w,direction)
		    /transformation	$selectfile($w,format)"
	    } else {
		Message:InvokeFunction ApplyShadowTransform "
		    /transformsource	($filename)
		    /Direction		$selectfile($w,direction)
		    /transformation	$selectfile($w,format)"
	    }
	}

        "SaveShadowTransform" {
            Message:InvokeFunction SaveShadowTransform "
                /filename               ($filename)"
            set result [ catch { exec sh -c " sleep 10 ; sed \"s/ShadowTransform/UserTransformation/g\" $filename > ${filename}TMP ; /bin/mv ${filename}TMP ${filename} ; tal=`grep talairachcalibrated $filename | grep true | wc -l` ; if \[ \$tal = 1 \] ; then sed \"s/UserTransformation/IntoTalairachSpace/g\" $filename > ${filename}TMP ; /bin/mv ${filename}TMP ${filename} ; fi" & } ErrMsg ]
            puts "transform file fixed"
        }

	"SelectHeadHatFile" {
	    Message:InvokeFunction SelectHeadHat "
		/FileName		($filename)"
	}

	default {
#	    puts "SelectFile:apply what function $selectfile($w,funcname)?"
	}
    }
    SetHistoryProperties [ $w.box subwidget filter subwidget listbox ] \
        Session $selectfile(history)
}


# Procedure:	SelectFile:format
#
# Usage:	SelectFile:format widget
#
# Summary:	
#
proc SelectFile:format {w format} {
    global selectfile
    
    set selectfile($w,format) $format

    if {$format == "RawGeneric"} {
	[$w.format subwidget label] config -width 10
	pack $w.format $w.rawspecs -in $w.ff -side left
    } else {
	pack forget $w.rawspecs
	[$w.format subwidget label] config -width 16
	pack $w.format -in $w.ff -side top
    }
}


# Procedure:	SelectFile:destroy
#
# Usage:	SelectFile:destroy widget
#
# Summary:	Called whenever the widget is to be destroyed.
#		Unsets all global variables associated with the widget
#		before destroying it.
#
proc SelectFile:destroy {w} {
    global selectfile
    global saveimagedir
    global saveimagelist
    global oldsaveformat oldsavegformat currenttype



    set saveimagedir [[[$w.box subwidget filter] subwidget entry] get]
#    set saveimagedir [string trimright $saveimagedir "*"]
#    set saveimagedir [string trimright $saveimagedir "/"]

    set histlist [[$w.box subwidget filter] subwidget listbox]
    set saveimagelist [GetListItems $histlist]

    Message:DialogUnposted $w

    if {$currenttype == "Save Image As"} {
	set oldsaveformat $selectfile($w,format)
    } elseif {$currenttype == "Save Group As"} {
	set oldsavegformat $selectfile($w,format)
    }
    unset selectfile($w,format)
    unset selectfile($w,disposition)
    catch { unset selectfile($w,cookie) }
    destroy $w
}


# This "if" statement makes it possible to run this script file inside or
# outside of the main demo program "phase?".
#
if {![info exists phase_demo_running]} {
    lappend auto_path ../
    lappend auto_path ./
    wm withdraw .
    set bitmapdir ../bitmaps
    set remote_out stdout
    OpenFolder .fi2 
}

