#include <stdlib.h>
#include <stdio.h>
#include "avwio.h"

size_t AvwReadVolumes(AVW *avw, void *buffer, size_t nvols)
{
  size_t volbytes;
  size_t n;
  short type;

  volbytes = AvwGetVolSize(avw) * (AvwGetDataType(avw, &type) / 
				   (8 * sizeof(char)));
  if((n=fread(buffer, volbytes, nvols, avw->imgfp)) != nvols)
    perror("AvwReadVolume");

  if(!avw->sameorder)
    AvwReverseBuffer(avw, buffer, AvwGetVolSize(avw) * nvols);

  return n;
}

size_t AvwReadSliceSeries(AVW *avw, void *buffer,short slice, size_t nvols)
{
  size_t slbytes,volbytes;
  size_t n;
  short x,y,z,v,type;

  AvwGetDim(avw,&x,&y,&z,&v);
  
  if(slice>=z)perror("AvwReadSlices");

  slbytes = x * y * (AvwGetDataType(avw, &type) / 
		     (8 * sizeof(char)));
  volbytes = slbytes * z;

  fseek(avw->imgfp, slbytes*slice, SEEK_CUR);

  for(n=0; n<nvols; n++){
    if(n>0)fseek(avw->imgfp, volbytes - slbytes, SEEK_CUR);
    if(fread((char *)buffer+n*slbytes, slbytes, 1, avw->imgfp) != 1)
      perror("AvwReadSlices");
  }

  fseek(avw->imgfp, slbytes*(z-slice-1), SEEK_CUR);

  if(!avw->sameorder)
    AvwReverseBuffer(avw, buffer, x * y * nvols);

  return n;

}

size_t AvwReadRowSeries(AVW *avw, void *buffer, short row, short slice, size_t nvols)
{
  size_t rowbytes,slbytes,volbytes;
  size_t n;
  short x,y,z,v,type;

  AvwGetDim(avw,&x,&y,&z,&v);

  if(slice>=z)perror("AvwReadRows");
  if(row>=y)perror("AvwReadRows");
  
  rowbytes = x * (AvwGetDataType(avw, &type) / 
		  (8 * sizeof(char)));
  slbytes = rowbytes * y;
  volbytes = slbytes * z;

  fseek(avw->imgfp, rowbytes*row + slbytes*slice, SEEK_CUR);

  for(n=0; n<nvols; n++){
    if(n>0)fseek(avw->imgfp, volbytes - rowbytes, SEEK_CUR);
    if(fread((char *)buffer+n*rowbytes, rowbytes, 1, avw->imgfp) != 1)
      perror("AvwReadRows");
  }

  fseek(avw->imgfp, (slbytes*(z-slice-1)) + ((y-row-1)*rowbytes), SEEK_CUR);

  if(!avw->sameorder)
    AvwReverseBuffer(avw, buffer, y * nvols);

  return n;

}

