/* {{{ Copyright etc. */

/*  avwroi.c - extract cuboid ROI from image

    Stephen Smith, FMRIB Image Analysis Group  */

/* }}} */
/* {{{ defines, includes and typedefs */

#include "libss.h"
#include "libavw.h"

void usage(void);

/* }}} */
/* {{{ usage */

void usage(void)
{
  fprintf(stderr,"Usage: avwroi <input> <output> <xmin> <xsize> <ymin> <ysize> <zmin> <zsize> [tmin tsize]\n");
  exit(1);
}

/* }}} */
/* {{{ main */

int main(argc,argv) 
     int argc;
     char **argv;
{
  FDT *out;
  image_struct im;
  int x,y,z,t,xmin,xsize,ymin,ysize,zmin,zsize,tmin,tsize;

  /* {{{ process args */

if (argc<9)
  usage();
    
avw_read(argv[1],&im);
im.t=MAX(im.t,1);
tmin=0;
tsize=im.t;

xmin=atoi(argv[3]);
xsize=atoi(argv[4]);
ymin=atoi(argv[5]);
ysize=atoi(argv[6]);
zmin=atoi(argv[7]);
zsize=atoi(argv[8]);

if (argc>9)
{
  tmin=atoi(argv[9]);
  tsize=atoi(argv[10]);
}

out=malloc(xsize*ysize*zsize*tsize*sizeof(FDT));  

/* }}} */
  /* {{{ process roi and output */

  for(t=0;t<tsize;t++)
    for(z=0;z<zsize;z++)
      for(y=0;y<ysize;y++)
	for(x=0;x<xsize;x++)
	  out[t*zsize*ysize*xsize + z*ysize*xsize + y*xsize + x] =
	    im.i[(t+tmin)*im.z*im.y*im.x + (z+zmin)*im.y*im.x + (y+ymin)*im.x + x+xmin];

  im.i=out;
  im.x=xsize;	  
  im.y=ysize;	  
  im.z=zsize;	  
  im.t=tsize;	  

  avw_write(argv[2],im);

/* }}} */

  exit(0);
}

/* }}} */
