#

#{{{ copyright

#   FEAT - FMRIB's Easy Analysis Tool for FMRI
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 1999-2000 University of Oxford
#
#   Part of FSL - FMRIB's Software Library
#   WWW:      http://www.fmrib.ox.ac.uk/fsl
#   Email:    fsl@fmrib.ox.ac.uk
#   
#   Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
#   Imaging of the Brain), Department of Clinical Neurology, Oxford
#   University, Oxford, UK
#   
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or (at
#   your option) any later version.
#   
#   This program is distributed in the hope that it will be useful, but in
#   order that the University as a charitable foundation protects its
#   assets for the benefit of its educational and research purposes, the
#   University makes clear that no condition is made or to be implied, nor
#   is any warranty given or to be implied, as to the accuracy of FSL, or
#   that it will be suitable for any particular purpose or for use under
#   any specific conditions.  Furthermore, the University disclaims all
#   responsibility for the use which is made of FSL.  See the GNU General
#   Public License for more details.
#   
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#   USA

#}}}

proc feat3_proc { featname popups FeatDirName highres } {
    #{{{ Setup variables

set thedate [exec date]
puts "Starting FEAT at $thedate"

global PXHOME FSLDIR HOME USER PROCID HOSTNAME OS VERSION MEDXV CLUSTERRSH

MxShowInformationalPopups 0
if { $MEDXV > 3.25 } {
    MxShowGasGauge 0
}

# set no image history and don't save old versions (which shouldn't now exist) to disk
MxSetImageHistoryDefaults 0 false

# set tile pool correctly if the user has a small one
MxGetTilePoolDefaults currentsize
if { $currentsize < 100 } {
    MxSetTilePoolDefaults 150
}

MxGetCurrentFolder F0
MxGetPageByName $F0 "FirstPage" FirstPage

#}}}
    #{{{ Read FEAT setup file

    set channel [ open ${featname}.fsf "r" ]

    gets $channel line
    scan $line "%f %d %d %d" fileversion type analysis paradigm_type
    gets $channel line
    scan $line "%f %d %d" tr npts ndelete
    gets $channel line
    scan $line "%d %f %d %d %d" multiple delay saveprep_yn savefolder_yn help_yn
    gets $channel line
    scan $line "%d %f %d %d %f %d %d %d %f" filtering_yn brain_thresh mc sh_yn smooth norm_yn temphp_yn templp_yn paradigm_hp
    gets $channel line
    scan $line "%d" stats_yn
    gets $channel line
    scan $line "%d %d %d %d %d" evs ncon dmred dmgrey constcol
    gets $channel line
    scan $line "%d %f %s" mf_type mf_cycle mf_file
    gets $channel line
    scan $line "%d %d %f %f" poststats_yn thresh prob_thresh z_thresh
    gets $channel line
    scan $line "%d %f %f %d" zdisplay zmin zmax rendertype

    if { $fileversion > 3.125 } {
	gets $channel line
	scan $line "%d %d %s %d" reg_yn regstandard_yn regstandard reghighres_yn
    } else {
	set reg_yn 0
    }

    close $channel

#}}}
    if { $analysis > 0 } {
	#{{{ Delete images

if { $analysis != 4 } {

    MxGetCurrentGroup Group
    MxGetPagesFromGroup $Group GPages
    set total_volumes [ llength $GPages ]
    puts "Total original volumes = $total_volumes"
    if { $total_volumes != $npts } {
	puts "Error - current group has different number of time points to what was entered"
	return 1
    }

    if { $ndelete > 0 } {
	puts "Deleting pages from loaded group - BE WARNED for future analysis!"
    }

    set i 0
    while { $i < $ndelete } {
	set Todelete [ lindex $GPages $i ]
	puts "Deleting page $Todelete"
	MxDeletePage { $Todelete }
	incr i 1
    }

    MxGetPagesFromGroup $Group GPages
    set total_volumes [ llength $GPages ]
    MxSetTimeSeriesProperties $Group 1 red

}

#}}}
	#{{{ Get group file name and setup folder details

if { $analysis == 4 } {

    set OldFeatDirName $FeatDirName

    set FeatDirName [ new_filename $OldFeatDirName ]
    
    if { ! [ file writable [ file dirname $FeatDirName ] ] } {
	set FeatDirName [ new_filename ${HOME}/[ file tail $OldFeatDirName ] ]
    }

    file copy $OldFeatDirName $FeatDirName
    exec sh -c "cd $FeatDirName ; rm -f Rendered_thresh_zstat* cluster_mask_zstat* cluster_zstat* design* prob_mask_zstat* report* thresh_zstat* tsplot* ; cd stats ; rm -f cope* neff* tstat* varcope* zstat*"

} else {
    MxGetPageProperties $Group GroupProps

    # find full filename and replace some unfriendly characters
    set GroupFullNameTmp [ keylget GroupProps Name ]
    regsub -all " " $GroupFullNameTmp "" GroupFullNameTmp2
    regsub -all "\"" $GroupFullNameTmp2 "" GroupFullName

    # find directory and filetailroot
    set GroupDirName [ file dirname  $GroupFullName ]
    regsub -all "\\." [ file rootname [ file tail $GroupFullName ] ] "_" GroupFileRoot

    set NotTmp [ string compare /tmp [ string range $GroupDirName 0 3 ] ]
    if { ! [ file writable $GroupDirName ] || $NotTmp == 0 } {
	set GroupName ${HOME}/${GroupFileRoot}
    } else {
	set GroupName ${GroupDirName}/${GroupFileRoot}
    }

    set FeatDirName [ new_filename ${GroupName}.feat ]
    exec /bin/mkdir $FeatDirName
}

set CLUSTER  ${FeatDirName}/cluster
set STATSDIR ${FeatDirName}/stats

set FOLDER   ${FeatDirName}/folder
MxSetFolderProperties $F0 "{Name ${FOLDER}} {File ${FOLDER}.fld} {SaveAll true}"

set DESIGN   ${FeatDirName}/design
exec /bin/cp ${featname}.fsf ${DESIGN}.fsf
if { $paradigm_type == 3 } {
    exec /bin/cp ${featname}.mat ${DESIGN}.mat
    exec /bin/cp ${featname}.con ${DESIGN}.con
    exec /bin/cp ${featname}.gif ${DESIGN}.gif
}

#}}}
	#{{{ Select target_vol

if { $analysis != 4 } {

    # choose image 10
    set target_vol_number 9

    # but not if there aren't 10 images in group
    if { $target_vol_number >= $total_volumes } {
	set target_vol_number [ expr $total_volumes - 1 ]
    }

    # but choose 0 if doing SPM MC (cos that's what it uses)
    if { $mc == 4 && $filtering_yn == 1 } {
	set target_vol_number 0
    }

    # now create target_vol, get its props, name it and move it to top of folder
    MxDupPage [ lindex $GPages $target_vol_number ] target_vol
    MxGetImageProperties $target_vol target_props
    name_page $target_vol "Example functional image"
    MxCutPage $target_vol
    MxPastePage $FirstPage target_vol

    FSLSaveAs $target_vol AVW ${FeatDirName}/example_func.hdr true

} else {

    MxOpenImage $F0 ${FeatDirName}/example_func.hdr target_vol
    MxGetImageProperties $target_vol target_props

}

#}}}
	#{{{ Write starts of log and report files

set LOGFILE    ${FeatDirName}/report.log
set REPORTFILE ${FeatDirName}/report.html
set logout [ open $LOGFILE "w" ]
set report [ open $REPORTFILE "w" ]

fconfigure $logout -buffering none
fconfigure $report -buffering none

puts $logout "Started FEAT at $thedate on $HOSTNAME"

puts $report "<HTML>

<TITLE>FEAT Report</TITLE>

<BODY BACKGROUND=\"file:${FSLDIR}/doc/images/fsl-bg\">

<hr><CENTER>
<H1>FEAT Report</H1>
$REPORTFILE
</CENTER>
"

set ps "<hr><H2>Analysis methods</H2>
<p>Analysis was carried out using FEAT, the FMRIB Easy Analysis Tool, an extension of MEDx \[MEDx\]."

set rs "<p><b>References</b><br>
\[MEDx\] Sensor Systems, VA, USA.<br>
"

#}}}
	#{{{ Pre-stats

#{{{ find threshold

# should really make this more robust

MxQuietStatistics $target_vol { Maximum Minimum } false Stats
set McMin [ keylget Stats Minimum ]
set McMax [ keylget Stats Maximum ]

set absbrainthresh [ expr ( ( $McMax - $McMin ) * $brain_thresh / 100.0 ) + $McMin ]

puts "Brain intensity threshold $absbrainthresh"
puts $logout "Brain intensity threshold $absbrainthresh"

#}}}

MxSetCurrentPage $target_vol

if { $analysis != 4 } {
    if { $filtering_yn } {

	set ps "$ps The following pre-statistics processing was applied"

	#{{{ Motion correction

#mc: 0=none 1=guess 2=2DAIR 3=3DAIR 4=SPM

if { $mc > 0 } {

    if { $mc == 1 } {
	#{{{ guess MC type from number of slices

if { [ keylget target_props Slices ] < 7 } {
    set mc 2
} else {
    set mc 3
}

#}}}
    }

    if { $mc == 2 } {
	#{{{ 2D AIR

	MxAlignLinear $target_vol $Group 23 "{Interpolation 1} {StdThr $absbrainthresh} {ResliceThr $absbrainthresh} {StdPartitions 1} {ReslicePartitions 1} {Convergence 1.0000000000000001e-05} {RepeatedIter 25} {HaltAfterIter 5} {AlterAfterIter 6} {InitSampling 81} {FinalSampling 1} {SamplingDecr 3} {StdXSmooth 0} {StdYSmooth 0} {StdZSmooth 0} {ResliceXSmooth 0} {ResliceYSmooth 0} {ResliceZSmooth 0} {Directory /tmp} {Cost stddev} {pre-interp false} {NonInteract true}" NewGroup
	set ps "$ps; 2D motion correction, using AIR \[AIR\]"
	set rs "$rs\[AIR\] R.P. Woods, S.T. Grafton, C.J. Holmes, S.R. Cherry and J.C. Mazziotta. Automated image registration: I. General methods and intrasubject, intramodality validation. Journal of Computer Assisted Tomography 1998:22(141-154).<br>
"

#}}}
    } elseif { $mc == 3 } {
	#{{{ 3D AIR

	MxAlignLinear $target_vol $Group 6 "{Interpolation 1} {StdThr $absbrainthresh} {ResliceThr $absbrainthresh} {StdPartitions 1} {ReslicePartitions 1} {Convergence 1.0000000000000001e-05} {RepeatedIter 25} {HaltAfterIter 5} {AlterAfterIter 6} {InitSampling 81} {FinalSampling 1} {SamplingDecr 3} {StdXSmooth 0} {StdYSmooth 0} {StdZSmooth 0} {ResliceXSmooth 0} {ResliceYSmooth 0} {ResliceZSmooth 0} {Directory /tmp} {Cost stddev} {pre-interp false} {NonInteract true}" NewGroup
	set ps "$ps; 3D motion correction, using AIR \[AIR\]"
	set rs "$rs\[AIR\] R.P. Woods, S.T. Grafton, C.J. Holmes, S.R. Cherry and J.C. Mazziotta. Automated image registration: I. General methods and intrasubject, intramodality validation. Journal of Computer Assisted Tomography 1998:22(141-154).<br>
"

#}}}
    } elseif { $mc == 4 } {
	#{{{ SPM

	set sh 0
	set shstring ""
	if { $sh_yn == 1 } {
	    if { $tr > 4 } {
		if { $total_volumes > 40 } {
		    set sh 2
		} elseif { $total_volumes > 30 } {
		    set sh 1
		}
	    } else {
		if { $total_volumes > 40 } {
		    set sh 3
		}
		if { $tr < 2 && $total_volumes > 64 } {
		    set sh 4
		}
	    }
	    set shstring "with adjustment for movement "
	}
	puts "SPM MC adjustment $sh"
	puts $logout "SPM MC adjustment $sh"
	
	keylset SPMProps CoregInterp 1
	keylset SPMProps ResliceInterp 1
	keylset SPMProps Reg_Reslice Both
	keylset SPMProps Output 1
	keylset SPMProps NumIterations 8
	keylset SPMProps Mask true
	keylset SPMProps UsePrevXForms false
	keylset SPMProps fMRIAdjust $sh
	keylset SPMProps ReportDir "/tmp"

	MxSPMRealign $Group SPMProps NewGroup

	set ps "$ps; motion correction using the SPM realign procedure $shstring\[SPM1\]"
	set rs "$rs\[SPM1\] K.J. Friston, J. Ashburner, J.-B. Poline, C.D. Frith, J.D. Heather and R.S.J. Frackowiak. Spatial Registration and Normalization of Images. Human Brain Mapping 1995:2(165-189).<br>
"

#}}}
    }

    set Group $NewGroup
    MxGetPagesFromGroup $Group GPages
    set total_volumes [ llength $GPages ]
    MxSetTimeSeriesProperties $Group 1 red
}

#}}}

	set ip_command ""
	#{{{ Spatial filtering

if { $smooth > 0.01 } {
    set ip_command "$ip_command -s [ expr $smooth / 2.355 ]"
    set ps "$ps; spatial smoothing using a Gaussian kernel of FWHM ${smooth}mm"
}

#}}}
	#{{{ Masking and global intensity normalization

set ip_command "$ip_command -m ${FeatDirName}/mask"

set global_mean 10000

if { $norm_yn} {
    set ip_command "$ip_command -i $global_mean"
    set ps "$ps; global (volumetric) multiplicative mean intensity renormalization"
} else {
    set ip_command "$ip_command -I $global_mean"
    set ps "$ps; mean-based intensity normalisation of all volumes by the same factor"
}

#}}}
	#{{{ Temporal filtering

if { $temphp_yn || $templp_yn } {

    set hp_sigma_vol -1
    if { $temphp_yn } {
	if { $paradigm_type == 4 } {
	    set paradigm_hp [ expr $mf_cycle * $tr * 1.5 ]
	}
	set hp_sigma_sec [ expr $paradigm_hp / 2.0 ]
	set hp_sigma_vol [ expr $hp_sigma_sec / $tr ]
	set ps "$ps; nonlinear highpass temporal filtering (Gaussian-weighted LSF straight line fitting, with sigma=${hp_sigma_sec}s)"
    }

    set lp_sigma_vol -1
    if { $templp_yn } {
	set lp_sigma_sec 2.8
	set lp_sigma_vol [ expr $lp_sigma_sec / $tr ]
	set ps "$ps; Gaussian lowpass temporal filtering HWHM ${lp_sigma_sec}s"
    }

    set ip_command "$ip_command -t $hp_sigma_vol $lp_sigma_vol"

}

#}}}
	#{{{ Call ip and reload data

set ps "$ps."

FSLSaveAs $Group AVW ${FeatDirName}/prefiltered_func_data.hdr true

puts "Filtering: $ip_command"
puts $logout "Filtering: $ip_command"

set absbrainthresh [ exec ${FSLDIR}/bin/ip ${FeatDirName}/prefiltered_func_data ${FeatDirName}/filtered_func_data $brain_thresh $ip_command | grep THRESHOLD | awk {{ print $2 }} - ]

exec /bin/rm -f ${FeatDirName}/prefiltered_func_data.hdr ${FeatDirName}/prefiltered_func_data.img

MxSetCurrentPage $target_vol

MxOpenImage $F0 ${FeatDirName}/filtered_func_data.hdr NewGroup

set Group $NewGroup
MxGetPagesFromGroup $Group GPages
set total_volumes [ llength $GPages ]
MxSetTimeSeriesProperties $Group 1 red

# following hack so that if the file is later deleted, MEDx still works
MxComputeDisplayRange [ lindex $GPages 0 ] V Min Max

#}}}

    } else {
	FSLSaveAs $Group AVW ${FeatDirName}/filtered_func_data.hdr true
    }
}

MxSetCurrentPage $target_vol

#}}}
	#{{{ Stats

if { $stats_yn || $analysis == 4 } {

    #{{{ load or generate binary mask

    if { [file exists ${FeatDirName}/mask.hdr] } {
	MxOpenImage $F0 ${FeatDirName}/mask.hdr Mask
	MxThreshold $Mask 0.5 B1
    } else {
	MxGenerateMask $Group $brain_thresh Mask
	MxThreshold $Mask 0.5 B1
	FSLSaveAs $Mask AVW ${FeatDirName}/mask.hdr true
    }

#}}}

    if { $analysis != 4 } {
	if { $paradigm_type == 3 } {
	    #{{{ Get FILM going

if { $type == 2 } {
    set film_opts "-sa -ms 5"
} else {
    set film_opts "-gl"
}

set thecommand "$CLUSTERRSH $FSLDIR/bin/film_ols -rn $STATSDIR $film_opts -sp $FSLDIR/bin/susan_smooth ${FeatDirName}/filtered_func_data ${DESIGN}.mat $absbrainthresh"
puts "Running $thecommand in the background"
set cpid [ open "| $thecommand" r ]
fconfigure $cpid -blocking false

#}}}
	}
	#{{{ Make fake Z score

set AList ""
lappend AList [ lindex $GPages 0 ]
lappend AList [ lindex $GPages 1 ]
MxGroupVolumes $AList "A" A

set BList ""
set i 2
while { $i < $total_volumes } {
    lappend BList [ lindex $GPages $i ]
    incr i 1
}
MxGroupVolumes $BList "B" B

MxBetweenGrpStatistics $A $B U blah blah2 blah3 FinalZscore ResultGroup

MxGetPagesFromGroup $ResultGroup RP
set RPList " [ lindex $RP 0 ] [ lindex $RP 1 ] [ lindex $RP 2 ] [ lindex $RP 3 ] [ lindex $RP 4 ] [ lindex $RP 6 ] [ lindex $RP 7 ] "
MxDeletePage { $RPList }
MxDeleteGroupPages { $A $B $ResultGroup } No

#}}}
    } else {
	#{{{ Make fake Z score

MxDupPage $target_vol dummy

MxGroupVolumes "$target_vol $dummy" "A" A

MxSetCurrentPage $Mask
MxBetweenGrpStatistics $A $A U blah blah2 blah3 FinalZscore ResultGroup

MxGetPagesFromGroup $ResultGroup RP
set RPList " [ lindex $RP 0 ] [ lindex $RP 1 ] [ lindex $RP 2 ] [ lindex $RP 3 ] [ lindex $RP 4 ] [ lindex $RP 6 ] [ lindex $RP 7 ] "
MxDeletePage { $RPList }
MxDeleteGroupPages { $A $ResultGroup $dummy } No

#}}}
    }

    if { $paradigm_type == 3 } {
	#{{{ Wait for FILM to finish and run contrasts

if { $analysis != 4 } {
    if { $popups } {
	ScriptUpdate "Running FILM - FMRIB's Improved Linear Model"
    }

    while { ! [ eof $cpid ] } {
	gets $cpid dummy
	after 5000
    }
    
    if { $popups } {
	CancelScriptUpdate
    }
}

set thecommand "exec $FSLDIR/bin/contrast_mgr $STATSDIR ${DESIGN}.con"
puts "$thecommand"
set result [ catch { eval $thecommand >& ${STATSDIR}.txt } ErrMsg ]

exec /bin/mv -f ${STATSDIR}.txt $STATSDIR

#}}}
	#{{{ load in results

set i $ncon
while { $i >= 1 } {
    MxOpenImage $F0 ${STATSDIR}/zstat${i}.hdr Tmp
    MxDupPage $FinalZscore ZS($i)
    MxExpr $ZS($i) = " $Tmp "
    MxComputeDisplayRange $ZS($i) V min max
    name_page $ZS($i) "zstat$i"

    MxOpenImage $F0 ${STATSDIR}/cope${i}.hdr Tmp2
    MxDupPage $FinalZscore PE($i)
    MxExpr $PE($i) = " $Tmp2 "
    MxComputeDisplayRange $PE($i) V min max
    name_page $PE($i) "cope$i"

    MxDeletePage { $Tmp $Tmp2 }
    incr i -1
}

MxDeletePage { $FinalZscore }

#}}}
	#{{{ paragraph output

set ps "$ps Statistical analysis was carried out using FILM (FMRIB's Improved Linear Model) with local autocorrelation correction \[FILM\]."
set rs "$rs\[FILM\] M. Woolrich, J.M. Brady, B. Ripley and S.M. Smith. Robust Treatment of Temporal Autocorrelations in Univariate Linear Modelling of FMRI Data. In preparation 2000.<br>
"

#}}}
    } else {
	set ncon 1
	#{{{ Call model-free

if { $analysis != 4 } {
    set ncon 1

    MxSetCurrentPage $Group

    if { $popups } {
	ScriptUpdate "Running model-free ANOVA"
    }

    exec /bin/mkdir $STATSDIR

    set dofout [ open ${STATSDIR}/dof "w" ]
    puts $dofout $total_volumes
    close $dofout

    if { $mf_type == 1 } {
	set thecommand "exec ${FSLDIR}/bin/medx_anova ${FeatDirName}/filtered_func_data ${STATSDIR}/fstat1 ${STATSDIR}/zstat1 $brain_thresh 0 [ expr int($mf_cycle) ]"
    } else {
	set mf_events [ exec wc -l $mf_file | awk {{ print $1 }} - ]
	puts "Found $mf_events events in model-free events file."
	set thecommand "exec ${FSLDIR}/bin/medx_anovanc ${FeatDirName}/filtered_func_data ${STATSDIR}/fstat1 ${STATSDIR}/zstat1 $brain_thresh $mf_events [ expr int($mf_cycle) ] $mf_file"
    }

    puts $thecommand
    set result [ catch { eval $thecommand >& ${STATSDIR}/mf.txt } ErrMsg ]

    if { $popups } {
	CancelScriptUpdate
    }
} else {
    exec /bin/cp -f ${OldFeatDirName}/stats/zstat1.hdr ${STATSDIR}/zstat1.hdr
    exec /bin/cp -f ${OldFeatDirName}/stats/zstat1.img ${STATSDIR}/zstat1.img
}

set paradigm_type 1

#}}}
	#{{{ load in results

MxOpenImage $F0 ${STATSDIR}/zstat1.hdr Tmp
MxDupPage $FinalZscore ZS(1)
MxExpr $ZS(1) = " $Tmp "
MxComputeDisplayRange $ZS(1) V min max
name_page $ZS(1) "zstat1"
MxDeletePage { $Tmp }
MxDeletePage { $FinalZscore }

#}}}
	#{{{ paragraph output

set ps "$ps Statistical analysis was carried out using the model-free ANOVA method \[ANOVA\]."
set rs "$rs\[ANOVA\] S. Clare, M. Humberstone, J. Hykin, L. Blumhardt, R. Bowtell and P. Morris. Detecting Activations in Event-Related fMRI Using Analysis of Variance. Magnetic Resonance in Medicine 1999:42(1117-1122).<br>
"

#}}}
    }
}

#}}}
	#{{{ Poststats

if { $poststats_yn } {
    #{{{ setup zmin and zmax to initial extremes if doing autoscale

    if { $zdisplay == 0 } {
	set zmin 1000
	set zmax -1000
    }

#}}}
    puts "thresholding is $thresh"
    puts $logout "thresholding is $thresh"
    for { set i 1 } { $i <= $ncon } { incr i 1 } {
	#{{{ mask Z stat images

MxExpr $ZS($i) = " ( $ZS($i) * $Mask ) "

#}}}
	#{{{ Thresholding

#{{{ get name of stats image

MxGetImageProperties $ZS($i) stats_props
set stats_name [keylget stats_props Name]
regsub -all " " $stats_name "_" stats_name_fixed($i)

puts "$stats_name:"
puts $logout "$stats_name:"

#}}}
#{{{ Find Z max

MxQuietStatistics $ZS($i) { Maximum } false Stats

set MaximumZ($i) [ keylget Stats Maximum ]

puts "Original MaximumZ =$MaximumZ($i)"
puts $logout "Original MaximumZ =$MaximumZ($i)"

#}}}
#{{{ save raw Z score to file if required

if { $paradigm_type == 4 } {
    FSLSaveAs $ZS($i) AVW ${FeatDirName}/raw_$stats_name_fixed($i).hdr true
}

#}}}

if { $thresh > 0 } {
    #{{{ paragraph output

if { $i == 1 } {
    if { $ncon == 1 } {
	set ps "$ps A Z (Gaussianised T) statistic image was thresholded"
    } else {
	set ps "$ps Z (Gaussianised T) statistic images were thresholded"
    }
}

#}}}
    #{{{ Duplicate Z stat image

    MxDupPage $ZS($i) ZS($i)
    name_page $ZS($i) "thresh_zstat$i"

#}}}

    if { $thresh == 1 } {
	# voxel uncorrected thresholding
	#{{{ Find corrected thresholds and paragraph output

if { $i == 1 } {
    set ps "$ps at P=${prob_thresh} (uncorrected)."

    if { $MEDXV == 3.2 } {
	set prob_thresh [expr $prob_thresh * 2]
    }

    if {$prob_thresh > 0.99} {
	set prob_thresh 0.99
    }
}

if { $MEDXV == 3.2 } {
    MxComputeUncorrectedCriticalThr $Mask U $prob_thresh dummy1 dummy2 zuncorrected dummy3
} else {
    MxComputeUncorrectedCriticalThr $Mask U 2 $prob_thresh dummy1 dummy2 zuncorrected
}

puts "zuncorrected=$zuncorrected"
puts $logout "zuncorrected=$zuncorrected"

#}}}
	#{{{ Use corrected Z threshold to mask Z image

MxMapPixels $ZS($i) -50 $zuncorrected 0 0

MxQuietStatistics $ZS($i) { Maximum } false Stats
set MaximumZ($i) [ keylget Stats Maximum ]
puts "MaximumZ =$MaximumZ($i) after thresholding"
puts $logout "MaximumZ =$MaximumZ($i) after thresholding"

#}}}
    } else {
	if { $thresh == 2 } {
	    # voxel resel corrected thresholding
	    #{{{ Paragraph output and fix prob_thresh

if { $i == 1 } {
    set ps "$ps using resel (corrected Bonferroni) thresholding with a corrected significance threshold of P=${prob_thresh} \[SPM2\]."
    set rs "$rs\[SPM2\] K.J. Friston, A.P. Holmes, K.J. Worsley, J.P. Poline, C.D. Frith and R.S.J. Frackowiak. Statistical Parametric Maps in Functional Imaging: A General Linear Approach. Human Brain Mapping 1995:2(189-210).<br>
"

    if { $MEDXV == 3.2 } {
	set prob_thresh [expr $prob_thresh * 2]
    }

    if {$prob_thresh > 0.99} {
	set prob_thresh 0.99
    }
}

#}}}
	    #{{{ Do the correction

if { $MEDXV == 3.2 } {
    MxClusterDetection $ZS($i) $Mask 1 0.5 "/tmp/${USER}${PROCID}_cluster.txt" I5($i) I6($i) I7($i)
    set X_Resels [exec grep FWHMx /tmp/${USER}${PROCID}_cluster.txt | grep PHYSICAL | awk {{ print $5 }} - ]
    set Y_Resels [exec grep FWHMy /tmp/${USER}${PROCID}_cluster.txt | grep PHYSICAL | awk {{ print $5 }} - ]
    set Z_Resels [exec grep FWHMz /tmp/${USER}${PROCID}_cluster.txt | grep PHYSICAL | awk {{ print $5 }} - ]
    puts "X_Resels=$X_Resels Y_Resels=$Y_Resels Z_Resels=$Z_Resels"
    puts $logout "X_Resels=$X_Resels Y_Resels=$Y_Resels Z_Resels=$Z_Resels"

    set errrr [catch {exec /bin/rm -f /tmp/${USER}${PROCID}_cluster.txt} junk]
    set errrr [catch {exec /bin/rm -f /tmp/${USER}${PROCID}_cluster.txt} junk]
    set errrr [catch {exec /bin/rm -f /tmp/${USER}${PROCID}_cluster.txt} junk]

    MxComputeCorrectedCriticalThr $Mask R $X_Resels $Y_Resels $Z_Resels $prob_thresh outputp zcorrected zuncorrected count
} else {
    MxComputeCorrectedCriticalThr $Mask RS $ZS($i) 2 $prob_thresh outputp zcorrected zuncorrected count
}

puts "outputp=$outputp zcorrected=$zcorrected zuncorrected=$zuncorrected count=$count"
puts $logout "outputp=$outputp zcorrected=$zcorrected zuncorrected=$zuncorrected count=$count"

#}}}
	    #{{{ Use corrected Z threshold to mask Z image

MxMapPixels $ZS($i) -50 $zcorrected 0 0

MxQuietStatistics $ZS($i) { Maximum } false Stats
set MaximumZ($i) [ keylget Stats Maximum ]
puts "MaximumZ =$MaximumZ($i) after thresholding"
puts $logout "MaximumZ =$MaximumZ($i) after thresholding"

#}}}
	} else {
	    # cluster thresholding
	    #{{{ paragraph preparation

if { $i == 1 } {
    set ps "$ps using clusters determined by Z>${z_thresh} and a cluster significance threshold of P=${prob_thresh} \[SPM3\]."
    set rs "$rs\[SPM3\] K.J. Friston, K.J. Worsley, R.S.J. Frackowiak, J.C. Mazziotta and A.C. Evans. Assessing the Significance of Focal Activations Using their Spatial Extent. Human Brain Mapping 1994:1(214-220).<br>
"
}

#}}}
	    if { $MaximumZ($i) > $z_thresh } {
		#{{{ Cluster detection

if { $MEDXV == 3.2 } {
    # for some reason editor popup doesn't work with full path name
    MxClusterDetection $ZS($i) $Mask $z_thresh $prob_thresh /tmp/${USER}${PROCID}_cluster.txt I5($i) I6($i) I7($i)
    set junk [exec /bin/mv /tmp/${USER}${PROCID}_cluster.txt ${CLUSTER}_$stats_name_fixed($i).txt]
} else {
    MxClusterDetection $ZS($i) $Mask S $ZS($i) $z_thresh $prob_thresh /tmp/${USER}${PROCID}_cluster.txt I5($i) I6($i) I7($i) 1
    set junk [exec /bin/mv /tmp/${USER}${PROCID}_cluster.txt ${CLUSTER}_$stats_name_fixed($i).txt]
}

#}}}
		#{{{ Use cluster mask to mask Z image

MxDupPage $I5($i) ClusterMask($i)
MxThreshold $ClusterMask($i) 0.5 B1
MxExpr $ZS($i) = " $ZS($i) * $ClusterMask($i) "

#}}}
	    } else {
		MxExpr $ZS($i) = 0
	    }
	    #{{{ report max Z after thresh

MxQuietStatistics $ZS($i) { Maximum } false Stats
set MaximumZ($i) [ keylget Stats Maximum ]
puts "MaximumZ =$MaximumZ($i) after thresholding"
puts $logout "MaximumZ =$MaximumZ($i) after thresholding"

#}}}
	    #{{{ save cluster and probability masks if required

if { $MaximumZ($i) > 0.02 } {
    FSLSaveAs $I5($i) AVW ${FeatDirName}/cluster_mask_$stats_name_fixed($i).hdr true
    FSLSaveAs $I6($i) AVW ${FeatDirName}/prob_mask_$stats_name_fixed($i).hdr true
}

#}}}
	}
    }
}

#}}}
	#{{{ Find group Z min and max

if { $MaximumZ($i) > 0 && $zdisplay == 0 } {
    
    MxQuietStatistics $ZS($i) { Minimum } false 0.002 Stats

    set zminl [ expr [ keylget Stats Minimum ] - 0.002 ]
    if { $zminl < 0.002 } {
	set zminl 0.002
    }

    if { $zmin > $zminl } {
	set zmin $zminl
    }

    if { $zmax < $MaximumZ($i) } {
	set zmax $MaximumZ($i)
    }
}

#}}}
	#{{{ save Z score to file

FSLSaveAs $ZS($i) AVW ${FeatDirName}/thresh_$stats_name_fixed($i).hdr true

#}}}
    }
    
    if { $thresh > 0 } {
	#{{{ Rendering

set RenderList ""
set rendered 0

MxQuietStatistics $target_vol { Minimum Maximum } false Stats
set origmin [ keylget Stats Minimum ]
set origmax [ keylget Stats Maximum ]

for { set i 1 } { $i <= $ncon } { incr i 1 } {
    #{{{ Colour rendering

if { $MaximumZ($i) > 0 } {
    incr rendered 1
    set thedate [exec date]
    puts "Starting rendering at $thedate"
    puts $logout "Starting rendering at $thedate"
        
    puts "Rendering using zmin=$zmin zmax=$zmax"
    puts $logout "Rendering using zmin=$zmin zmax=$zmax"
    
    render_proc $F0 $rendertype 0 1 "$target_vol $origmin $origmax $ZS($i) $zmin $zmax"
    
    MxGetCurrentPage R($i)
    lappend RenderList $R($i)

    MxGetImageProperties $R($i) render_props
    set render_name [keylget render_props Name]
    regsub -all " " $render_name "_" render_name_fixed
        
    FSLSaveAs $R($i) AVW ${FeatDirName}/${render_name_fixed}.hdr true

    print_gif [ file rootname [ new_filename ${FeatDirName}/${render_name_fixed}.gif ] ]
    
    #{{{ make report page links

if { $i == 1 } {
    puts $report "<hr><H2>Thresholded activation images</H2>"
}

if { $thresh == 3 } {
    puts $report "<p>Contrast $i<br>
    <a href=\"cluster_$stats_name_fixed($i).txt\"><IMG BORDER=0 SRC=\"${render_name_fixed}.gif\" WIDTH=\"600\"></a>
    "
} else {
    puts $report "<p>Contrast $i<br>
    <IMG BORDER=0 SRC=\"${render_name_fixed}.gif\" WIDTH=\"600\">
    "
}

#}}}
}

#}}}
}

if { $rendered > 1 } {
    MxSetCurrentPage $FirstPage
    MxGroupVolumes { $RenderList } "Rendered Z statistics" FinalGroup
}

#}}}
    }
}

#}}}
	#{{{ Time series plots and report output for each contrast

if { $paradigm_type == 3 } {
    set fmrifile ${FeatDirName}/filtered_func_data
    if { ! [ file exists ${fmrifile}.hdr ] } {
	set fmrifile [ string trimright [ file root ${FeatDirName} ] + ]
    }

    set thecommand "exec ${FSLDIR}/bin/tsplot ${FeatDirName} -f $fmrifile"
    puts $thecommand
    set result [ catch { eval $thecommand } ErrMsg ]
}

if { [ file exists ${FeatDirName}/tsplot1.gif ] } {
    puts $report "<hr><H2>Time series plots</H2><p>"
    for { set i 1 } { $i <= $ncon } { incr i 1 } {
	puts $report "<a href=\"tsplot${i}p.gif\"><IMG BORDER=0 SRC=\"tsplot${i}.gif\"></a><br>
	"

	if { [ file exists ${FeatDirName}/tsplotc${i}.gif ] } {
	    puts $report "<a href=\"tsplotc${i}p.gif\"><IMG BORDER=0 SRC=\"tsplotc${i}.gif\"></a><br>
	    "
	}
    }
}

#}}}
	#{{{ Save folder

set thedate [exec date]
puts "Finished processing at $thedate"
puts $logout "Finished processing at $thedate"

if { $analysis != 4 && !$saveprep_yn } {
    exec /bin/rm -f ${FeatDirName}/filtered_func_data.hdr ${FeatDirName}/filtered_func_data.img
}

if { $savefolder_yn } {
    MxSaveFolder $F0
}

#}}}
    } else {
	#{{{ Write starts of log and report files

set DESIGN   ${FeatDirName}/flirtdesign
exec /bin/cp ${featname}.fsf ${DESIGN}.fsf

set LOGFILE    ${FeatDirName}/flirtreport.log
set REPORTFILE ${FeatDirName}/flirtreport.html
set logout [ open $LOGFILE "w" ]
set report [ open $REPORTFILE "w" ]

fconfigure $logout -buffering none
fconfigure $report -buffering none

puts $logout "Started FEAT at $thedate on $HOSTNAME"

puts $report "<HTML>

<TITLE>FLIRT Registration Report</TITLE>

<BODY BACKGROUND=\"file:${FSLDIR}/doc/images/fsl-bg\">

<hr><CENTER>
<H1>FLIRT Registration Report</H1>
$REPORTFILE<br>
for FEAT session<br>
<a href=\"${FeatDirName}/report.html\">${FeatDirName}</a>
</CENTER>
"

set ps "<hr><H2>Analysis methods</H2>
<p>Analysis was carried out using FEAT, the FMRIB Easy Analysis Tool, an extension of MEDx \[MEDx\]."

set rs "<p><b>References</b><br>
\[MEDx\] Sensor Systems, VA, USA.<br>
"

#}}}
    }
    #{{{ Registration

# could move to start and run both in parallel 

if { $reg_yn } {

    #{{{ report output

puts $report "<hr><H2>Registration results</H2>"

set ps "$ps Registration to high resolution and/or standard images was carried out using FLIRT \[FLIRT\]."
set rs "$rs\[FLIRT\] M. Jenkinson and S.M. Smith. Global Optimisation for Robust Affine Registration. Medical Image Analysis. In submission. 2000.<br>
"

#}}}
    #{{{ short names for different files

    set ef ${FeatDirName}/example_func
    set hr [ file rootname $highres ]
    set st [ file rootname $regstandard ]
    set ef2hr ${FeatDirName}/example_func2highres
    set hr2st ${FeatDirName}/highres2standard
    set ef2st ${FeatDirName}/example_func2standard
    set sl ${FeatDirName}/sliceroutput

#}}}
    #{{{ check FOV and set DOF

set MINFOV 120
set DOF 12

set xfov [ expr [ exec $FSLDIR/bin/avwval ${ef} pixdim1 ] * [ exec $FSLDIR/bin/avwval ${ef} dim1 ] ]
set yfov [ expr [ exec $FSLDIR/bin/avwval ${ef} pixdim2 ] * [ exec $FSLDIR/bin/avwval ${ef} dim2 ] ]
set zfov [ expr [ exec $FSLDIR/bin/avwval ${ef} pixdim3 ] * [ exec $FSLDIR/bin/avwval ${ef} dim3 ] ]

puts "Field of view of functional image = ($xfov,$yfov,$zfov)"

if { $xfov < $MINFOV || $yfov < $MINFOV || $zfov < $MINFOV } {
    puts "Thus reducing functional-structural registration DOF to 7"
    set DOF 7
}

#}}}
    #{{{ func -> highres

if { $reghighres_yn } {

    if { [file exists ${FeatDirName}/highres.hdr] } {
	exec rm -f ${FeatDirName}/highres.hdr
    }
    if { [file exists ${FeatDirName}/highres.img] } {
	exec rm -f ${FeatDirName}/highres.img
    }
    exec ln -f -s ${hr}.hdr ${FeatDirName}/highres.hdr
    exec ln -f -s ${hr}.img ${FeatDirName}/highres.img
    
    set command "exec $CLUSTERRSH ${FSLDIR}/bin/flirt -ref $hr -in $ef -out $ef2hr -omedx ${ef2hr}.xfm -omat ${ef2hr}.mat -dof $DOF -bins 256 -cost corratio -anglerep euler -nosave -searchrx -90 90 -searchry -90 90 -searchrz -90 90"
    ScriptUpdate "Running FLIRT registration of FMRI to high resolution image"
    puts $command
    set result [ catch { eval $command } ErrMsg ]
    CancelScriptUpdate

    set command "exec sh -c \" ${FSLDIR}/bin/slicer $ef2hr $hr -s 1 -x 0.4 ${sl}a -x 0.5 ${sl}b -x 0.6 ${sl}c -y 0.4 ${sl}d -y 0.5 ${sl}e -y 0.6 ${sl}f -z 0.4 ${sl}g -z 0.5 ${sl}h -z 0.6 ${sl}i ; ${FSLDIR}/bin/convert -colors 100 +append ${sl}a ${sl}b ${sl}c ${sl}d ${sl}e ${sl}f ${sl}g ${sl}h ${sl}i ${ef2hr}.gif ; /bin/rm -f ${sl}a ${sl}b ${sl}c ${sl}d ${sl}e ${sl}f ${sl}g ${sl}h ${sl}i \" "
    puts $command
    set result [ catch { eval $command } ErrMsg ]

    puts $report "<p>FMRI example image to high resolution image<br>
    <a href=\"example_func2highres.mat\"><IMG BORDER=0 SRC=\"example_func2highres.gif\"></a>
    "
}

#}}}
    #{{{ standard regs

if { $regstandard_yn } {

    if { [file exists ${FeatDirName}/standard.hdr] } {
	exec rm -f ${FeatDirName}/standard.hdr
    }
    if { [file exists ${FeatDirName}/standard.img] } {
	exec rm -f ${FeatDirName}/standard.img
    }
    exec ln -f -s ${st}.hdr ${FeatDirName}/standard.hdr
    exec ln -f -s ${st}.img ${FeatDirName}/standard.img

    if { $reghighres_yn } {
	set command "exec $CLUSTERRSH ${FSLDIR}/bin/flirt -ref $st -in $hr -out $hr2st -omedx ${hr2st}.xfm -omat ${hr2st}.mat -dof 12 -bins 256 -cost corratio -anglerep euler -nosave -searchrx -90 90 -searchry -90 90 -searchrz -90 90"
	ScriptUpdate "Running FLIRT registration of high resolution image to standard image"
	puts $command
	set result [ catch { eval $command } ErrMsg ]
	CancelScriptUpdate
	TalairachMedxTransform ${hr2st}.xfm

	set command "exec sh -c \" ${FSLDIR}/bin/slicer $hr2st $st -s 1 -x 0.4 ${sl}a -x 0.5 ${sl}b -x 0.6 ${sl}c -y 0.4 ${sl}d -y 0.5 ${sl}e -y 0.6 ${sl}f -z 0.4 ${sl}g -z 0.5 ${sl}h -z 0.6 ${sl}i ; ${FSLDIR}/bin/convert -colors 100 +append ${sl}a ${sl}b ${sl}c ${sl}d ${sl}e ${sl}f ${sl}g ${sl}h ${sl}i ${hr2st}.gif ; /bin/rm -f ${sl}a ${sl}b ${sl}c ${sl}d ${sl}e ${sl}f ${sl}g ${sl}h ${sl}i \" "
	puts $command
	set result [ catch { eval $command } ErrMsg ]

	puts $report "<p>High resolution image to standard image<br>
	<a href=\"highres2standard.mat\"><IMG BORDER=0 SRC=\"highres2standard.gif\"></a>
	"

	set command "exec ${FSLDIR}/bin/convert_xfm -ref $st -in $ef -omedx ${ef2st}.xfm -omat ${ef2st}.mat -concat ${hr2st}.xfm -middlevol $hr ${ef2hr}.xfm"
	puts $command
	set result [ catch { eval $command } ErrMsg ]
	TalairachMedxTransform ${ef2st}.xfm

	set command "exec $CLUSTERRSH ${FSLDIR}/bin/flirt -ref $st -in $ef -out $ef2st -init ${ef2st}.mat -applyxfm"
	puts $command
	set result [ catch { eval $command } ErrMsg ]
	
	set command "exec sh -c \" ${FSLDIR}/bin/slicer $ef2st $st -s 1 -x 0.4 ${sl}a -x 0.5 ${sl}b -x 0.6 ${sl}c -y 0.4 ${sl}d -y 0.5 ${sl}e -y 0.6 ${sl}f -z 0.4 ${sl}g -z 0.5 ${sl}h -z 0.6 ${sl}i ; ${FSLDIR}/bin/convert -colors 100 +append ${sl}a ${sl}b ${sl}c ${sl}d ${sl}e ${sl}f ${sl}g ${sl}h ${sl}i ${ef2st}.gif ; /bin/rm -f ${sl}a ${sl}b ${sl}c ${sl}d ${sl}e ${sl}f ${sl}g ${sl}h ${sl}i \" "
	puts $command
	set result [ catch { eval $command } ErrMsg ]

	puts $report "<p>FMRI example image to standard image<br>
	<a href=\"example_func2standard.mat\"><IMG BORDER=0 SRC=\"example_func2standard.gif\"></a>
	"
    } else {
	set command "exec $CLUSTERRSH ${FSLDIR}/bin/flirt -ref $st -in $ef -out $ef2st -omedx ${ef2st}.xfm -omat ${ef2st}.mat -dof $DOF -bins 256 -cost corratio -anglerep euler -nosave -searchrx -90 90 -searchry -90 90 -searchrz -90 90"
	ScriptUpdate "Running FLIRT registration of FMRI to standard image"
	puts $command
	set result [ catch { eval $command } ErrMsg ]
	CancelScriptUpdate
	TalairachMedxTransform ${ef2st}.xfm
	
	set command "exec sh -c \" ${FSLDIR}/bin/slicer $ef2st $st -s 1 -x 0.4 ${sl}a -x 0.5 ${sl}b -x 0.6 ${sl}c -y 0.4 ${sl}d -y 0.5 ${sl}e -y 0.6 ${sl}f -z 0.4 ${sl}g -z 0.5 ${sl}h -z 0.6 ${sl}i ; ${FSLDIR}/bin/convert -colors 100 +append ${sl}a ${sl}b ${sl}c ${sl}d ${sl}e ${sl}f ${sl}g ${sl}h ${sl}i ${ef2st}.gif ; /bin/rm -f ${sl}a ${sl}b ${sl}c ${sl}d ${sl}e ${sl}f ${sl}g ${sl}h ${sl}i \" "
	puts $command
	set result [ catch { eval $command } ErrMsg ]
	
	puts $report "<p>FMRI example image to standard image<br>
	<a href=\"example_func2standard.mat\"><IMG BORDER=0 SRC=\"example_func2standard.gif\"></a>
	"
    }
}

#}}}

    exec /bin/rm -f ${ef2hr}.hdr ${ef2hr}.img ${hr2st}.hdr ${hr2st}.img ${ef2st}.hdr ${ef2st}.img

} else {

    if { $analysis == 4 && [ file exists ${OldFeatDirName}/example_func2standard.mat ] } {
	puts $report "<hr>To see registration look <a href=\"${OldFeatDirName}/report.html\">here</a>."
    } else {
	puts $report "<hr>Registration was not carried out initially - later registration results will appear <a href=\"flirtreport.html\">here</a>."
    }

}

#}}}
    #{{{ Timing and final outputs

MxShowInformationalPopups 1
if { $MEDXV > 3.25 } {
    MxShowGasGauge 1
}

set thedate [exec date]

puts "Finished FEAT at $thedate"
puts $logout "Finished FEAT at $thedate"

puts $report "$ps

$rs

"
if { $paradigm_type == 3 } {
    puts $report "<p><p><b>Design matrix</a><br><a href=\"design.mat\"><IMG BORDER=0 SRC=\"design.gif\"></a>
    "
}

puts $report "<HR><FONT SIZE=1>This page produced automatically by FEAT $VERSION - a part of <A HREF=\"http://www.fmrib.ox.ac.uk/fsl\">FSL - FMRIB's Software Library</A>.</FONT>

</BODY></HTML>
"

close $logout
close $report

puts "
To view the FEAT report point your web browser at
$REPORTFILE"

return ${FeatDirName}

#}}}
}

proc feat3_gproc { featname popups } {
    #{{{ Setup variables

set thedate [exec date]
puts "Starting Group FEAT at $thedate"

global PXHOME FSLDIR HOME USER PROCID HOSTNAME OS VERSION MEDXV CLUSTERRSH

MxShowInformationalPopups 0
if { $MEDXV > 3.25 } {
    MxShowGasGauge 0
}

MxGetCurrentFolder F0
MxGetPageByName $F0 "FirstPage" FirstPage

#}}}
    #{{{ Read FEAT setup file

    set channel [ open ${featname}.fsf "r" ]

    gets $channel line
    scan $line "%f %d %d %d" fileversion type analysis paradigm_type
    gets $channel line
    scan $line "%f %d %d" tr npts ndelete
    gets $channel line
    scan $line "%d %f %d %d %d" multiple delay saveprep_yn savefolder_yn help_yn
    gets $channel line
    scan $line "%d %f %d %d %f %d %d %d %f" filtering_yn brain_thresh mc sh_yn smooth norm_yn temphp_yn templp_yn paradigm_hp
    gets $channel line
    scan $line "%d" stats_yn
    gets $channel line
    scan $line "%d %d %d %d %d" evs ncon dmred dmgrey constcol
    gets $channel line
    scan $line "%d %f %s" mf_type mf_cycle mf_file
    gets $channel line
    scan $line "%d %d %f %f" poststats_yn thresh prob_thresh z_thresh
    gets $channel line
    scan $line "%d %f %f %d" zdisplay zmin zmax rendertype
    gets $channel line
    scan $line "%d %d %s %d" reg_yn regstandard_yn regstandard reghighres_yn
    gets $channel line
    scan $line "%d %d" group_yn ngroupa

    set i 1
    while { $i <= $multiple } {
	gets $channel line
	scan $line "%s" feat_files($i)
	incr i 1
    }

    if { $reg_yn && $reghighres_yn } {
	if { $multiple < 2 } {
	    set nhighres 1
	} else {
	    set nhighres $multiple
	}
	set i 1
	while { $i <= $nhighres } {
	    gets $channel line
	    scan $line "%s" highres_files($i)
	    incr i 1
	}
    }

    close $channel

#}}}
    #{{{ Get group file name and setup folder details

puts " $feat_files(1) "

set FeatDirName [ new_filename [ file rootname $feat_files(1) ].gfeat ]

if { ! [ file writable [ file dirname $FeatDirName ] ] } {
    set FeatDirName [ new_filename ${HOME}/[ file rootname [ file tail $feat_files(1) ] ].gfeat ]
}

exec /bin/mkdir $FeatDirName

set FOLDER ${FeatDirName}/folder
MxSetFolderProperties $F0 "{Name ${FOLDER}} {File ${FOLDER}.fld} {SaveAll true}"

set DESIGN ${FeatDirName}/design
exec /bin/cp ${featname}.fsf ${DESIGN}.fsf

set CLUSTER  ${FeatDirName}/cluster

set STATSDIR ${FeatDirName}/stats
exec /bin/mkdir ${STATSDIR}

#}}}
    #{{{ Write starts of log and report files

set LOGFILE    ${FeatDirName}/report.log
set REPORTFILE ${FeatDirName}/report.html
set logout [ open $LOGFILE "w" ]
set report [ open $REPORTFILE "w" ]

fconfigure $logout -buffering none
fconfigure $report -buffering none

puts $logout "Started Group FEAT at $thedate on $HOSTNAME"

puts $report "<HTML>

<TITLE>Group FEAT Report</TITLE>

<BODY BACKGROUND=\"file:${FSLDIR}/doc/images/fsl-bg\">

<hr><CENTER>
<H1>Group FEAT Report</H1>
$REPORTFILE
</CENTER>
"

set ps "<hr><H2>Analysis methods</H2>
<p>Group (fixed and random effects) analysis was carried out using FEAT, the FMRIB Easy Analysis Tool, an extension of MEDx \[MEDx\]."

set rs "<p><b>References</b><br>
\[MEDx\] Sensor Systems, VA, USA.<br>
"

#}}}
    #{{{ Links to first-level reports

puts $report "<hr><H2>First-level reports</H2>"

if { $ngroupa>0 } {
    puts $report "<b>Group A</b> (control):<br>
"
}

for { set i 1 } { $i <= $multiple } { incr i 1 } {

if { $ngroupa>0 && $i==$ngroupa+1 } {
    puts $report "<b>Group B</b> (test):<br>
"
}

    puts $report "FEAT report ${i}: <A HREF=\"$feat_files($i)/report.html\">$feat_files($i)/report.html</A><br>
"
    
}

#}}}
    #{{{ Do the group stats

set featdirlist ""
for { set i 1 } { $i <= $multiple } { incr i 1 } {
    set featdirlist "$featdirlist $feat_files($i)"
}

set command "exec $CLUSTERRSH ${FSLDIR}/bin/gfeat $ngroupa [ expr $multiple - $ngroupa ] $featdirlist ${STATSDIR} > ${STATSDIR}.txt"
ScriptUpdate "Running Group FEAT stats"
puts $command
set result [ catch { eval $command } ErrMsg ]
CancelScriptUpdate

# fix headers
exec sh -c " cd $FeatDirName ; for ff in stats/fixed*.hdr stats/random*.hdr ; do cp -f stats/.hdr \$ff ; done "

#}}}
    #{{{ load mask and highres

MxOpenImage $F0 ${STATSDIR}/mask.hdr Mask

if { [ file exists ${STATSDIR}/mean_highres.hdr ] } {
    MxOpenImage $F0 ${STATSDIR}/mean_highres.hdr target_vol
} else {
    MxOpenImage $F0 $feat_files(1)/standard.hdr target_vol
}

MxQuietStatistics $target_vol { Minimum Maximum } false Stats
set origmin [ keylget Stats Minimum ]
set origmax [ keylget Stats Maximum ]

MxGetDimensions $target_vol width height slices

#}}}
    #{{{ threshold and render

puts "thresholding is $thresh"
puts $logout "thresholding is $thresh"

#{{{ Make fake Z score

MxDupPage $target_vol dummy

MxGroupVolumes "$target_vol $dummy" "A" A

MxSetCurrentPage $Mask
MxBetweenGrpStatistics $A $A U blah blah2 blah3 FinalZscore ResultGroup

MxGetPagesFromGroup $ResultGroup RP
set RPList " [ lindex $RP 0 ] [ lindex $RP 1 ] [ lindex $RP 2 ] [ lindex $RP 3 ] [ lindex $RP 4 ] [ lindex $RP 6 ] [ lindex $RP 7 ] "
MxDeletePage { $RPList }
MxDeleteGroupPages { $A $ResultGroup $dummy } No

#}}}
#{{{ setup zmin and zmax to initial extremes if doing autoscale

if { $zdisplay == 0 } {
    set zmin 1000
    set zmax -1000
}

#}}}
#{{{ find out how many contrasts

for { set ncon 1 } { [ file exists ${STATSDIR}/fixed${ncon}.hdr ] } { incr ncon 1 } {

}

incr ncon -1

#}}}

foreach type { fixed random } {
    for { set i $ncon } { $i > 0 } { incr i -1 } {
	#{{{ load in results

MxOpenImage $F0 ${STATSDIR}/${type}${i}.hdr Tmp

MxDupPage $FinalZscore ZS(${type}${i})
MxExpr $ZS(${type}${i}) = " $Tmp "
MxComputeDisplayRange $ZS(${type}${i}) V min max
name_page $ZS(${type}${i}) "${type}$i"

MxDeletePage { $Tmp }

#}}}
	#{{{ Thresholding

#{{{ get name of stats image

MxGetImageProperties $ZS(${type}${i}) stats_props
set stats_name [keylget stats_props Name]
regsub -all " " $stats_name "_" stats_name_fixed($i)

puts "$stats_name:"
puts $logout "$stats_name:"

#}}}
#{{{ Find Z max

MxQuietStatistics $ZS(${type}${i}) { Maximum } false Stats

set MaximumZ(${type}${i}) [ keylget Stats Maximum ]

puts "Original MaximumZ =$MaximumZ(${type}${i})"
puts $logout "Original MaximumZ =$MaximumZ(${type}${i})"

#}}}

if { $thresh > 0 } {
    #{{{ paragraph output

if { $i==1 && $type=="fixed" } {
    if { $ncon == 1 } {
	set ps "$ps A Z (Gaussianised T) statistic image was thresholded"
    } else {
	set ps "$ps Z (Gaussianised T) statistic images were thresholded"
    }
}

#}}}
    #{{{ Duplicate Z stat image

    MxDupPage $ZS(${type}${i}) ZS(${type}${i})
    name_page $ZS(${type}${i}) "thresh_${type}${i}"

#}}}

    if { $thresh == 1 } {
	# voxel uncorrected thresholding
	#{{{ Find corrected thresholds and paragraph output

if { $i==1 && $type=="fixed" } {
    set ps "$ps at P=${prob_thresh} (uncorrected)."

    if { $MEDXV == 3.2 } {
	set prob_thresh [expr $prob_thresh * 2]
    }

    if {$prob_thresh > 0.99} {
	set prob_thresh 0.99
    }
}

if { $MEDXV == 3.2 } {
    MxComputeUncorrectedCriticalThr $Mask U $prob_thresh dummy1 dummy2 zuncorrected dummy3
} else {
    MxComputeUncorrectedCriticalThr $Mask U 2 $prob_thresh dummy1 dummy2 zuncorrected
}

puts "zuncorrected=$zuncorrected"
puts $logout "zuncorrected=$zuncorrected"

#}}}
	#{{{ Use corrected Z threshold to mask Z image

MxMapPixels $ZS(${type}${i}) -50 $zuncorrected 0 0

MxQuietStatistics $ZS(${type}${i}) { Maximum } false Stats
set MaximumZ(${type}${i}) [ keylget Stats Maximum ]
puts "MaximumZ =$MaximumZ(${type}${i}) after thresholding"
puts $logout "MaximumZ =$MaximumZ(${type}${i}) after thresholding"

#}}}
    } else {
	if { $thresh == 2 } {
	    # voxel resel corrected thresholding
	    #{{{ Paragraph output and fix prob_thresh

if { $i==1 && $type=="fixed" } {
    set ps "$ps using resel (corrected Bonferroni) thresholding with a corrected significance threshold of P=${prob_thresh} \[SPM2\]."
    set rs "$rs\[SPM2\] K.J. Friston, A.P. Holmes, K.J. Worsley, J.P. Poline, C.D. Frith and R.S.J. Frackowiak. Statistical Parametric Maps in Functional Imaging: A General Linear Approach. Human Brain Mapping 1995:2(189-210).<br>
"

    if { $MEDXV == 3.2 } {
	set prob_thresh [expr $prob_thresh * 2]
    }

    if {$prob_thresh > 0.99} {
	set prob_thresh 0.99
    }
}

#}}}
	    #{{{ Do the correction

if { $MEDXV == 3.2 } {
    MxClusterDetection $ZS(${type}${i}) $Mask 1 0.5 "/tmp/${USER}${PROCID}_cluster.txt" I5(${type}${i}) I6(${type}${i}) I7(${type}${i})
    set X_Resels [exec grep FWHMx /tmp/${USER}${PROCID}_cluster.txt | grep PHYSICAL | awk {{ print $5 }} - ]
    set Y_Resels [exec grep FWHMy /tmp/${USER}${PROCID}_cluster.txt | grep PHYSICAL | awk {{ print $5 }} - ]
    set Z_Resels [exec grep FWHMz /tmp/${USER}${PROCID}_cluster.txt | grep PHYSICAL | awk {{ print $5 }} - ]
    puts "X_Resels=$X_Resels Y_Resels=$Y_Resels Z_Resels=$Z_Resels"
    puts $logout "X_Resels=$X_Resels Y_Resels=$Y_Resels Z_Resels=$Z_Resels"

    set errrr [catch {exec /bin/rm -f /tmp/${USER}${PROCID}_cluster.txt} junk]
    set errrr [catch {exec /bin/rm -f /tmp/${USER}${PROCID}_cluster.txt} junk]
    set errrr [catch {exec /bin/rm -f /tmp/${USER}${PROCID}_cluster.txt} junk]

    MxComputeCorrectedCriticalThr $Mask R $X_Resels $Y_Resels $Z_Resels $prob_thresh outputp zcorrected zuncorrected count
} else {
    MxComputeCorrectedCriticalThr $Mask RS $ZS(${type}${i}) 2 $prob_thresh outputp zcorrected zuncorrected count
}

puts "outputp=$outputp zcorrected=$zcorrected zuncorrected=$zuncorrected count=$count"
puts $logout "outputp=$outputp zcorrected=$zcorrected zuncorrected=$zuncorrected count=$count"

#}}}
	    #{{{ Use corrected Z threshold to mask Z image

MxMapPixels $ZS(${type}${i}) -50 $zcorrected 0 0

MxQuietStatistics $ZS(${type}${i}) { Maximum } false Stats
set MaximumZ(${type}${i}) [ keylget Stats Maximum ]
puts "MaximumZ =$MaximumZ(${type}${i}) after thresholding"
puts $logout "MaximumZ =$MaximumZ(${type}${i}) after thresholding"

#}}}
	} else {
	    # cluster thresholding
	    #{{{ paragraph preparation

if { $i==1 && $type=="fixed" } {
    set ps "$ps using clusters determined by Z>${z_thresh} and a cluster significance threshold of P=${prob_thresh} \[SPM3\]."
    set rs "$rs\[SPM3\] K.J. Friston, K.J. Worsley, R.S.J. Frackowiak, J.C. Mazziotta and A.C. Evans. Assessing the Significance of Focal Activations Using their Spatial Extent. Human Brain Mapping 1994:1(214-220).<br>
"
}

#}}}
	    if { $MaximumZ(${type}${i}) > $z_thresh } {
		#{{{ Cluster detection

if { $MEDXV == 3.2 } {
    # for some reason editor popup doesn't work with full path name
    MxClusterDetection $ZS(${type}${i}) $Mask $z_thresh $prob_thresh /tmp/${USER}${PROCID}_cluster.txt I5(${type}${i}) I6(${type}${i}) I7(${type}${i})
    set junk [exec /bin/mv /tmp/${USER}${PROCID}_cluster.txt ${CLUSTER}_$stats_name_fixed($i).txt]
} else {
    MxClusterDetection $ZS(${type}${i}) $Mask S $ZS(${type}${i}) $z_thresh $prob_thresh /tmp/${USER}${PROCID}_cluster.txt I5(${type}${i}) I6(${type}${i}) I7(${type}${i}) 1
    set junk [exec /bin/mv /tmp/${USER}${PROCID}_cluster.txt ${CLUSTER}_$stats_name_fixed($i).txt]
}

#}}}
		#{{{ Use cluster mask to mask Z image

MxDupPage $I5(${type}${i}) ClusterMask(${type}${i})
MxThreshold $ClusterMask(${type}${i}) 0.5 B1
MxExpr $ZS(${type}${i}) = " $ZS(${type}${i}) * $ClusterMask(${type}${i}) "

#}}}
	    } else {
		MxExpr $ZS(${type}${i}) = 0
	    }
	    #{{{ report max Z after thresh

MxQuietStatistics $ZS(${type}${i}) { Maximum } false Stats
set MaximumZ(${type}${i}) [ keylget Stats Maximum ]
puts "MaximumZ =$MaximumZ(${type}${i}) after thresholding"
puts $logout "MaximumZ =$MaximumZ(${type}${i}) after thresholding"

#}}}
	    #{{{ save cluster and probability masks if required

if { $MaximumZ(${type}${i}) > 0.02 } {
    FSLSaveAs $I5(${type}${i}) AVW ${FeatDirName}/cluster_mask_$stats_name_fixed($i).hdr true
    FSLSaveAs $I6(${type}${i}) AVW ${FeatDirName}/prob_mask_$stats_name_fixed($i).hdr true
}

#}}}
	}
    }
}

#}}}
	#{{{ Find group Z min and max

if { $MaximumZ(${type}${i}) > 0 && $zdisplay == 0 } {
    
    MxQuietStatistics $ZS(${type}${i}) { Minimum } false 0.002 Stats

    set zminl [ expr [ keylget Stats Minimum ] - 0.002 ]
    if { $zminl < 0.002 } {
	set zminl 0.002
    }

    if { $zmin > $zminl } {
	set zmin $zminl
    }

    if { $zmax < $MaximumZ(${type}${i}) } {
	set zmax $MaximumZ(${type}${i})
    }
}

#}}}
	#{{{ save Z score to file

FSLSaveAs $ZS(${type}${i}) AVW ${FeatDirName}/thresh_$stats_name_fixed($i).hdr true

#}}}
    }
}

#{{{ rendering

if { $thresh > 0 } {

    set RenderList ""
    set rendered 0

    puts $report "<hr><H2>Thresholded activation images</H2>"

    foreach type { fixed random } {
	for { set i 1 } { $i <= $ncon } { incr i 1 } {
	    #{{{ Colour rendering

if { $MaximumZ(${type}${i}) > 0 } {
    incr rendered 1
    set thedate [exec date]
    puts "Starting rendering at $thedate"
    puts $logout "Starting rendering at $thedate"
        
    puts "Rendering using zmin=$zmin zmax=$zmax"
    puts $logout "Rendering using zmin=$zmin zmax=$zmax"
    
    render_proc $F0 $rendertype 0 1 "$target_vol $origmin $origmax $ZS(${type}${i}) $zmin $zmax"
    
    MxGetCurrentPage R(${type}${i})
    lappend RenderList $R(${type}${i})

    MxGetImageProperties $R(${type}${i}) render_props
    set render_name [keylget render_props Name]
    regsub -all " " $render_name "_" render_name_fixed
        
    set command "exec sh -c \" ${FSLDIR}/bin/convert -colors 100 -crop 0x0 +append"
    for { set j [ expr $slices / 10 ] } { $j < [ expr ( $slices * 9 ) / 10 ] } { incr j [ expr $slices / 20 ] } {
	MxSetPrinterSetup "{Type Tiff} {Name ${FeatDirName}/sltmp${j}.gif}"
	MxGetPageByName $F0 "${render_name}/Slice${j}" PICCY
	MxPrintPage $PICCY
	set command "$command ${FeatDirName}/sltmp${j}.gif"
    }
    set command "$command ${FeatDirName}/${render_name_fixed}.gif ; /bin/rm -f ${FeatDirName}/sltmp* \" "
    puts $command
    set result [ catch { eval $command } ErrMsg ]
    
    FSLSaveAs $R(${type}${i}) AVW ${FeatDirName}/${render_name_fixed}.hdr true

    if { [ file exists ${FeatDirName}/Rendered_thresh_${type}${i}.gif ] } {
	puts $report "<p>Contrast $i $type effects<br>
	<a href=\"cluster_${type}${i}.txt\"><IMG BORDER=0 SRC=\"Rendered_thresh_${type}${i}.gif\"></a>
	"
    }
}

#}}}
	}
    }

    MxSetCurrentPage $FirstPage
    if { $rendered > 1 } {
	MxGroupVolumes { $RenderList } "Rendered Z statistics" FinalGroup
    }
}

#}}}

MxDeletePage { $FinalZscore }

# fix headers
exec sh -c " cd $FeatDirName ; for ff in *.hdr ; do ${FSLDIR}/bin/avwcpgeom stats/.hdr \$ff ; done "

#}}}
    #{{{ Save folder

set thedate [exec date]
puts "Finished processing at $thedate"
puts $logout "Finished processing at $thedate"

if { $savefolder_yn } {
    MxSaveFolder $F0
}

#}}}
    #{{{ Timing and final outputs

MxShowInformationalPopups 1
if { $MEDXV > 3.25 } {
    MxShowGasGauge 1
}

set thedate [exec date]

puts "Finished Group FEAT at $thedate"
puts $logout "Finished Group FEAT at $thedate"

puts $report "$ps

$rs

"

if { $paradigm_type != 4 } {
    puts $report "<p><p><b>Design matrix</a><br><a href=\"$feat_files(1)/design.mat\"><IMG BORDER=0 SRC=\"$feat_files(1)/design.gif\"></a>
    "
}

puts $report "<HR><FONT SIZE=1>This page produced automatically by FEAT $VERSION - a part of <A HREF=\"http://www.fmrib.ox.ac.uk/fsl\">FSL - FMRIB's Software Library</A>.</FONT>

</BODY></HTML>
"

close $logout
close $report

puts "
To view the Group FEAT report point your web browser at
${FeatDirName}/report.html"

#}}}
}
