#

#   concat_transforms - concatenate two registration transofmrations
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 1999-2000 University of Oxford
#
#   Part of FSL - FMRIB's Software Library
#   WWW:      http://www.fmrib.ox.ac.uk/fsl
#   Email:    fsl@fmrib.ox.ac.uk
#   
#   Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
#   Imaging of the Brain), Department of Clinical Neurology, Oxford
#   University, Oxford, UK
#   
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or (at
#   your option) any later version.
#   
#   This program is distributed in the hope that it will be useful, but in
#   order that the University as a charitable foundation protects its
#   assets for the benefit of its educational and research purposes, the
#   University makes clear that no condition is made or to be implied, nor
#   is any warranty given or to be implied, as to the accuracy of FSL, or
#   that it will be suitable for any particular purpose or for use under
#   any specific conditions.  Furthermore, the University disclaims all
#   responsibility for the use which is made of FSL.  See the GNU General
#   Public License for more details.
#   
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#   USA

if { [ info exists env(FSLDIR) ] } {
    set FSLDIR $env(FSLDIR)
} else {
    set FSLDIR $PXHOME/fsl
}

# Summary:      Creates a file select dialog box.
proc ConcTrans { w } {
    global ConcTrans
 
    if [winfo exists $w] {
        wm deiconify $w
        raise $w
        return
    }
 
    toplevel $w
    wm title $w "Concat two transforms"
    wm iconname $w "Concat two transforms"
 
    frame $w.f
 
    tixFileEntry $w.filename  -label "First Transform:"\
        -variable ConcTrans($w,filename) \
        -options {
            entry.width 40
        }
    $w.filename filedialog subwidget fsbox config -directory ~
    pack $w.filename -in $w.f -side top -expand yes -fill both \
        -padx 3 -pady 10

    tixOptionMenu $w.format -label "First Transform Type:" \
        -variable ConcTrans($w,format) \
        -options {
            label.anchor w
            menubutton.width 20
    }
    OpMenu $w.format [list Talairach AIR Reslice Resample User]
    set ConcTrans($w,format) AIR
    $w.format configure -value $ConcTrans($w,format)
    pack $w.format -in $w.f -side top -padx 3 -pady 3

    tixFileEntry $w.filename2  -label "Second Transform:"\
        -variable ConcTrans($w,filename2) \
        -options {
            entry.width 40
        }
    $w.filename2 filedialog subwidget fsbox config -directory ~
    pack $w.filename2 -in $w.f -side top -expand yes -fill both \
        -padx 3 -pady 10

    tixOptionMenu $w.format2 -label "Second Transform Type:" \
        -variable ConcTrans($w,format2) \
        -options {
            label.anchor w
            menubutton.width 20
    }
    OpMenu $w.format2 [list Talairach AIR Reslice Resample User]
    set ConcTrans($w,format2) Talairach
    $w.format2 configure -value $ConcTrans($w,format2)
    pack $w.format2 -in $w.f -side top -padx 3 -pady 3

    tixFileEntry $w.filename3  -label "Output Transform:"\
        -variable ConcTrans($w,filename3) \
        -options {
            entry.width 40
        }
    $w.filename3 filedialog subwidget fsbox config -directory ~
    pack $w.filename3 -in $w.f -side top -expand yes -fill both \
        -padx 3 -pady 10

    # ---- Status Bar ----
 
    label $w.status -height 1 -relief sunken -bd 2
 
    #------------- Button Frame --------------

    frame $w.btns
    frame $w.btns.b -relief raised -borderwidth 1
 
    button $w.ok \
        -text "OK" -width 5 \
        -command "ConcTrans:apply $w destroy"
    bind $w.ok <Return> {
        [winfo toplevel %W].ok invoke}
 
    button $w.apply     -command "ConcTrans:apply $w keep" \
        -text "Apply" -width 5
    bind $w.apply <Return> {
        [winfo toplevel %W].apply invoke}
 
    button $w.cancel    -command "ConcTrans:destroy $w" \
        -text "Cancel" -width 5
    bind $w.cancel <Return> {
        [winfo toplevel %W].cancel invoke}
 
    pack $w.btns.b -side bottom -fill x -padx 3 -pady 5
    pack $w.ok $w.apply $w.cancel -in $w.btns.b \
        -side left -expand yes -padx 3 -pady 10 -fill y
 
    pack $w.f $w.status $w.btns -expand yes -fill both
}
 
 
#
# Procedure:    ConcTrans:apply
#
# Usage:        ConcTrans:apply widget destroy/keep
#
# Summary:      Creates a file select dialog box.
#
proc ConcTrans:apply { w dialog } {
    global ConcTrans

    $w.filename update
    $w.filename2 update
    $w.filename3 update

    set result [MxConcatTransforms $ConcTrans($w,filename) $ConcTrans($w,format) \
                    $ConcTrans($w,filename2) $ConcTrans($w,format2) $ConcTrans($w,filename3)]

    if {$result} {
      MxPause "Transform types are incorrect for the chosen transforms"
    } else {
      # Let the user know were done
      $w.status configure -text "Done"
      update idletasks

      # Get rid of the dialog box if OK button used
      if {$dialog == "destroy"} {
          ConcTrans:destroy $w
      }
    }
}

# Summary:      Destroys ConcTrans dialog box
proc ConcTrans:destroy { w } {
 
        destroy $w
}

ConcTrans .rename
tkwait window .rename

