/* {{{ Copyright etc. */

/*  create_lut - create MEDx LUTs and material maps for colour rendering

    Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 1999-2000 University of Oxford  */

/*  Part of FSL - FMRIB's Software Library
    WWW:      http://www.fmrib.ox.ac.uk/fsl
    Email:    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.
    
    This program is distributed in the hope that it will be useful, but in
    order that the University as a charitable foundation protects its
    assets for the benefit of its educational and research purposes, the
    University makes clear that no condition is made or to be implied, nor
    is any warranty given or to be implied, as to the accuracy of FSL, or
    that it will be suitable for any particular purpose or for use under
    any specific conditions.  Furthermore, the University disclaims all
    responsibility for the use which is made of FSL.  See the GNU General
    Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA */

/* }}} */
/* {{{ includes and defines */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

/* }}} */
/* {{{ main() */

int main(argc,argv)
  int argc;
  char *argv[];
{
  /* {{{ variables */

FILE   *lut1,*map1,*lut2,*map2,*lut1t,*map1t,*lut2t,*map2t;
int    i1=0, i2=0, j, k;
double a, intensity=0, red, green, blue;
char   filename[1000];

/* }}} */

  /* {{{ usage */

  if (argc<2)
  {
    printf("Usage: create_lut <output_file_root>\n");
    exit(1);
  }

/* }}} */
  /* {{{ prepare output files */

sprintf(filename,"%s1.lut",argv[1]);  lut1=fopen(filename,"w");
sprintf(filename,"%s1.map",argv[1]);  map1=fopen(filename,"w");

sprintf(filename,"%s2.lut",argv[1]);  lut2=fopen(filename,"w");
sprintf(filename,"%s2.map",argv[1]);  map2=fopen(filename,"w");

sprintf(filename,"%s1t.lut",argv[1]); lut1t=fopen(filename,"w");
sprintf(filename,"%s1t.map",argv[1]); map1t=fopen(filename,"w");

sprintf(filename,"%s2t.lut",argv[1]); lut2t=fopen(filename,"w");
sprintf(filename,"%s2t.map",argv[1]); map2t=fopen(filename,"w");

/* }}} */

  /* {{{ Starting text for luts and mmaps */

fprintf(lut1,"%%!VEST-LUT\n%%%%BeginInstance\n<<\n/SavedInstanceClassName /ClassLUT \n/PseudoColorMinimum 0.00 \n/PseudoColorMaximum 1.00 \n/PseudoColorMinControl /Low \n/PseudoColorMaxControl /High \n/PseudoColormap [\n");
fprintf(map1,"%%!VEST-MaterialMap\n%%%%BeginInstance\n<<\n/SavedInstanceClassName /ClassMaterialMap \n/Name (Render Map) \n/Colors [\n");

fprintf(lut2,"%%!VEST-LUT\n%%%%BeginInstance\n<<\n/SavedInstanceClassName /ClassLUT \n/PseudoColorMinimum 0.00 \n/PseudoColorMaximum 1.00 \n/PseudoColorMinControl /Low \n/PseudoColorMaxControl /High \n/PseudoColormap [\n");
fprintf(map2,"%%!VEST-MaterialMap\n%%%%BeginInstance\n<<\n/SavedInstanceClassName /ClassMaterialMap \n/Name (Render Map) \n/Colors [\n");

fprintf(lut1t,"%%!VEST-LUT\n%%%%BeginInstance\n<<\n/SavedInstanceClassName /ClassLUT \n/PseudoColorMinimum 0.00 \n/PseudoColorMaximum 1.00 \n/PseudoColorMinControl /Low \n/PseudoColorMaxControl /High \n/PseudoColormap [\n");
fprintf(map1t,"%%!VEST-MaterialMap\n%%%%BeginInstance\n<<\n/SavedInstanceClassName /ClassMaterialMap \n/Name (Render Map) \n/Colors [\n");

fprintf(lut2t,"%%!VEST-LUT\n%%%%BeginInstance\n<<\n/SavedInstanceClassName /ClassLUT \n/PseudoColorMinimum 0.00 \n/PseudoColorMaximum 1.00 \n/PseudoColorMinControl /Low \n/PseudoColorMaxControl /High \n/PseudoColormap [\n");
fprintf(map2t,"%%!VEST-MaterialMap\n%%%%BeginInstance\n<<\n/SavedInstanceClassName /ClassMaterialMap \n/Name (Render Map) \n/Colors [\n");

/* }}} */
  /* {{{ colour maps for luts and mmaps */

  /* {{{ type 1 */

for (a=0; a<1; a+=0.01)
{
  fprintf(lut1,"<-color{%f,%f,%f}->\n",a,a,a);
  fprintf(map1,"<-color{%f,%f,%f}->\n",a,a,a);
  i1++;
}
for (a=0; a<1; a+=0.01)
{
  fprintf(lut1,"<-color{1.0,%f,0.0}->\n",a);
  fprintf(map1,"<-color{1.0,%f,0.0}->\n",a);
  i1++;
}

/* }}} */
  /* {{{ type 2 */

for (a=0; a<1; a+=0.01)
{
  fprintf(lut2,"<-color{%f,%f,%f}->\n",a,a,a);
  fprintf(map2,"<-color{%f,%f,%f}->\n",a,a,a);
  i2++;
}
for (a=0; a<1; a+=0.01)
{
  fprintf(lut2,"<-color{1.0,%f,0.0}->\n",a);
  fprintf(map2,"<-color{1.0,%f,0.0}->\n",a);
  i2++;
}
for (a=0; a<1; a+=0.01)
{
  fprintf(lut2,"<-color{0.0,%f,1.0}->\n",a);
  fprintf(map2,"<-color{0.0,%f,1.0}->\n",a);
  i2++;
}
for (a=0; a<1; a+=0.005)
{
  fprintf(lut2,"<-color{0.0,%f,0.0}->\n",a/2+0.5);
  fprintf(map2,"<-color{0.0,%f,0.0}->\n",a/2+0.5);
  i2++;
}

/* }}} */

#define STARTINT 0.25
#define K1 5
#define K2 10
#define K3 20

  /* {{{ type 1t */

for (a=0; a<1; a+=0.01)
{
  fprintf(lut1t,"<-color{%f,%f,%f}->\n",a,a,a);
  fprintf(map1t,"<-color{%f,%f,%f}->\n",a,a,a);
}

for(j=0; j<5; j++)
  for(k=0; k<K3; k++)
  {
    if (k<K1)              intensity = STARTINT;
    if ((k>=K1) && (k<K2)) intensity = ((double)k-K1)/((K2-K1-1)/(1-STARTINT)) + STARTINT;
    if (k>=K2)             intensity = 1;

    red=1.0 * intensity;
    green=((double)j)/4.0 * intensity;

    fprintf(lut1t,"<-color{%f,%f,0.0}->\n",red,green);
    fprintf(map1t,"<-color{%f,%f,0.0}->\n",red,green);
  }

/* }}} */
  /* {{{ type 2t */

for (a=0; a<1; a+=0.01)
{
  fprintf(lut2t,"<-color{%f,%f,%f}->\n",a,a,a);
  fprintf(map2t,"<-color{%f,%f,%f}->\n",a,a,a);
}

for(j=0; j<5; j++)
  for(k=0; k<K3; k++)
  {
    if (k<K1)              intensity = STARTINT;
    if ((k>=K1) && (k<K2)) intensity = ((double)k-K1)/((K2-K1-1)/(1-STARTINT)) + STARTINT;
    if (k>=K2)             intensity = 1;

    red=1.0 * intensity;
    green=((double)j)/4.0 * intensity;

    fprintf(lut2t,"<-color{%f,%f,0.0}->\n",red,green);
    fprintf(map2t,"<-color{%f,%f,0.0}->\n",red,green);
  }

for(j=0; j<5; j++)
  for(k=0; k<K3; k++)
  {
    if (k<K1)              intensity = STARTINT;
    if ((k>=K1) && (k<K2)) intensity = ((double)k-K1)/((K2-K1-1)/(1-STARTINT)) + STARTINT;
    if (k>=K2)             intensity = 1;

    blue=1.0 * intensity;
    green=((double)j)/4.0 * intensity;

    fprintf(lut2t,"<-color{0.0,%f,%f}->\n",green,blue);
    fprintf(map2t,"<-color{0.0,%f,%f}->\n",green,blue);
  }

for(j=0; j<5; j++)
  for(k=0; k<K3*2; k++)
  {
    if (k<K1*2)                intensity = STARTINT;
    if ((k>=K1*2) && (k<K2*2)) intensity = ((double)k-K1*2)/((K2*2-K1*2-1)/(1-STARTINT)) + STARTINT;
    if (k>=K2*2)               intensity = 1;

    green=(((double)j)/8.0+0.5) * intensity;

    fprintf(lut2t,"<-color{0.0,%f,0.0}->\n",green);
    fprintf(map2t,"<-color{0.0,%f,0.0}->\n",green);
  }

/* }}} */

/* }}} */
  /* {{{ rest of material map */

fprintf(map1,"]\n/Names [\n");
fprintf(map2,"]\n/Names [\n");
fprintf(map1t,"]\n/Names [\n");
fprintf(map2t,"]\n/Names [\n");

for (j=0; j<i1; j++)
{
  fprintf(map1,"(Matrl%d)\n",j);
  fprintf(map1t,"(Matrl%d)\n",j);
}
for (j=0; j<i2; j++)
{
  fprintf(map2,"(Matrl%d)\n",j);
  fprintf(map2t,"(Matrl%d)\n",j);
} 
 
fprintf(map1,"]\n/MinValues [\n");
fprintf(map2,"]\n/MinValues [\n");
fprintf(map1t,"]\n/MinValues [\n");
fprintf(map2t,"]\n/MinValues [\n");

for (j=0; j<i1; j++)
{
  fprintf(map1,"%f\n",((double)j)*32000.0/((double)i1));
  fprintf(map1t,"%f\n",((double)j)*32000.0/((double)i1));
}
for (j=0; j<i2; j++)
{
  fprintf(map2,"%f\n",((double)j)*32000.0/((double)i2));
  fprintf(map2t,"%f\n",((double)j)*32000.0/((double)i2));
} 
 
fprintf(map1,"]\n/MaxValues [\n");
fprintf(map2,"]\n/MaxValues [\n");
fprintf(map1t,"]\n/MaxValues [\n");
fprintf(map2t,"]\n/MaxValues [\n");

for (j=1; j<=i1; j++)
{
  fprintf(map1,"%f\n",((double)j)*32000.0/((double)i1));
  fprintf(map1t,"%f\n",((double)j)*32000.0/((double)i1));
}
for (j=1; j<=i2; j++)
{
  fprintf(map2,"%f\n",((double)j)*32000.0/((double)i2));
  fprintf(map2t,"%f\n",((double)j)*32000.0/((double)i2));
}

fprintf(map1,"]\n/Opacities [\n");
fprintf(map2,"]\n/Opacities [\n");
fprintf(map1t,"]\n/Opacities [\n");
fprintf(map2t,"]\n/Opacities [\n");

for (j=0; j<i1/2; j++)
{
  fprintf(map1,"%f\n",((double)j)*0.05/(0.5*(double)i1)+0.05);
  fprintf(map1t,"%f\n",((double)j)*0.05/(0.5*(double)i1)+0.05);
}
for (j=i1/2; j<i1; j++)
{
  fprintf(map1,"0.1\n");
  fprintf(map1t,"0.1\n");
}

for (j=0; j<i2/5; j++)
{
  fprintf(map2,"%f\n",((double)j)*0.05/(0.2*(double)i2)+0.05);
  fprintf(map2t,"%f\n",((double)j)*0.05/(0.2*(double)i2)+0.05);
}
for (j=i2/5; j<i2; j++)
{
  fprintf(map2,"0.1\n");
  fprintf(map2t,"0.1\n");
}

fprintf(map1,"]\n/RenderMaterial? [\n");
fprintf(map2,"]\n/RenderMaterial? [\n");
fprintf(map1t,"]\n/RenderMaterial? [\n");
fprintf(map2t,"]\n/RenderMaterial? [\n");

for (j=1; j<=i1; j++)
{
  fprintf(map1,"true\n");
  fprintf(map1t,"true\n");
}
for (j=1; j<=i2; j++)
{
  fprintf(map2,"true\n");
  fprintf(map2t,"true\n");
}

/* }}} */
  /* {{{ end texts for luts and mmaps */

  fprintf(lut1,"]\n>>\n\n%%%%EndInstance\n%%%%EOF\n");
  fprintf(map1,"]\n>>\n\n%%%%EndInstance\n%%%%EOF\n");

  fprintf(lut2,"]\n>>\n\n%%%%EndInstance\n%%%%EOF\n");
  fprintf(map2,"]\n>>\n\n%%%%EndInstance\n%%%%EOF\n");

  fprintf(lut1t,"]\n>>\n\n%%%%EndInstance\n%%%%EOF\n");
  fprintf(map1t,"]\n>>\n\n%%%%EndInstance\n%%%%EOF\n");

  fprintf(lut2t,"]\n>>\n\n%%%%EndInstance\n%%%%EOF\n");
  fprintf(map2t,"]\n>>\n\n%%%%EndInstance\n%%%%EOF\n");

/* }}} */

  return(0);
}

/* }}} */
