#

#   iso - this changes the voxel size to make the image isotropic
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 1999-2000 University of Oxford
#
#   Part of FSL - FMRIB's Software Library
#   WWW:      http://www.fmrib.ox.ac.uk/fsl
#   Email:    fsl@fmrib.ox.ac.uk
#   
#   Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
#   Imaging of the Brain), Department of Clinical Neurology, Oxford
#   University, Oxford, UK
#   
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or (at
#   your option) any later version.
#   
#   This program is distributed in the hope that it will be useful, but in
#   order that the University as a charitable foundation protects its
#   assets for the benefit of its educational and research purposes, the
#   University makes clear that no condition is made or to be implied, nor
#   is any warranty given or to be implied, as to the accuracy of FSL, or
#   that it will be suitable for any particular purpose or for use under
#   any specific conditions.  Furthermore, the University disclaims all
#   responsibility for the use which is made of FSL.  See the GNU General
#   Public License for more details.
#   
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#   USA

if { [ info exists env(FSLDIR) ] } {
    set FSLDIR $env(FSLDIR)
} else {
    set FSLDIR $PXHOME/fsl
}

source ${FSLDIR}/tcl/medxstart.tcl
set auto_path [ linsert $auto_path 0 [ file dirname [ info script ] ] ]

MxGetCurrentFolder Folder

MxGetFolderProperties $Folder FolderProps

set FolderName [ keylget FolderProps Name ]
set FolderFile [ keylget FolderProps File ]
set FolderDirFile [ file dirname $FolderFile ]
set FolderTailFile [ file tail $FolderFile ]
set FolderExtension [ file extension $FolderFile ]

set IsTmp [ string compare /tmp [ string range $FolderDirFile 0 3 ] ]
if { ! [ file writable "${FolderDirFile}" ] || $IsTmp == 0 } {
    set FolderDirFile "/usr/people/${USER}"
}

set IsFld [ string compare .fld [ string range $FolderExtension 0 3 ] ]
if { $IsFld != 0 } {
    set FolderTailFile ${PROCID}.fld
}

MxSetFolderProperties $Folder "{File ${FolderDirFile}/${FolderTailFile}}"

set Continue [YesNoWidget "The folder will be saved to ${FolderDirFile}/${FolderTailFile} - is this OK?" "Yes" "No"]

if { ! $Continue } {
    MxPause "Change the folder file name in folder properties and then rerun this script"
    MxDialogFolderProperties 0
    exit
}

global Delay
set Delay 1.0
InputFromWidget "Delay" 1 "Enter delay in hours before saving folder" Delay 6

set Die [YesNoWidget "Do you want to kill MEDx after saving the folder?" "Yes" "No"]

after [ expr int( $Delay * 60.0 * 60.0 * 1000.0 ) ]

MxSaveFolder $Folder

if { $Die } {
    MxExitMEDx
}
