#

#{{{ copyright etc.

#   susan_proc.tcl - TCL call for susan smoothing
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 1999-2000 University of Oxford
#
#   Part of FSL - FMRIB's Software Library
#   WWW:      http://www.fmrib.ox.ac.uk/fsl
#   Email:    fsl@fmrib.ox.ac.uk
#   
#   Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
#   Imaging of the Brain), Department of Clinical Neurology, Oxford
#   University, Oxford, UK
#   
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or (at
#   your option) any later version.
#   
#   This program is distributed in the hope that it will be useful, but in
#   order that the University as a charitable foundation protects its
#   assets for the benefit of its educational and research purposes, the
#   University makes clear that no condition is made or to be implied, nor
#   is any warranty given or to be implied, as to the accuracy of FSL, or
#   that it will be suitable for any particular purpose or for use under
#   any specific conditions.  Furthermore, the University disclaims all
#   responsibility for the use which is made of FSL.  See the GNU General
#   Public License for more details.
#   
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#   USA

#}}}

proc susan_proc { input name bt output dt dim um usans usanlist } {

    global FSLDIR USER PROCID INMEDX

    #{{{ prepare thecommand

    set thecommand "exec $FSLDIR/bin/susan_smooth $input $bt $output $dt $dim $um $usans"

    set i 1
    while { $i <= $usans } {
	set thecommand "$thecommand [ lindex $usanlist [ expr ( $i - 1 ) * 2 ] ] [ lindex $usanlist [ expr ( ( $i - 1 ) * 2 ) + 1 ] ]"
	incr i 1
    }

    if { $usans > 0 } {
	set thecommand "$thecommand ${input}_usan_size"
    }

#}}}
    #{{{ call program

if { $INMEDX } {
    ScriptUpdate "Calling $thecommand\nAs this process is not fully integrated into MEDx, the traffic lights will not show up red during processing. Wait for this popup to disappear before continuing."
}

puts $thecommand

if { [ catch { eval $thecommand } ErrMsg ] != 0 } {
    puts "$ErrMsg"
}

puts Finished

if { $INMEDX } {
    CancelScriptUpdate
}

#}}}
    #{{{ load stuff into MEDx

if { $INMEDX } {

    MxGetCurrentFolder Folder

    if { $usans > 0 } {
        MxOpenImage $Folder ${input}_usan_size.hdr US
        MxGetImageProperties $US USProps
        keylset USProps Name "USAN size for SUSAN smoothed ${name}"
        MxSetImageProperties $US $USProps
	MxComputeDisplayRange $US umin umax
    }

    MxOpenImage $Folder ${output}.hdr SUSAN
    MxGetImageProperties $SUSAN SUSANProps
    keylset SUSANProps Name "SUSAN smoothed ${name}"
    MxSetImageProperties $SUSAN $SUSANProps
    MxComputeDisplayRange $SUSAN umin umax

}

#}}}
}
